/*****************************************************************************
 Copyright  Newcastle University
 Author      Paul Coates
 *****************************************************************************/
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <stdio.h>
#include "main.h"
#include "misc.h"
#include "router.h"
#include "network.h"
#include "interface.h"
#include "images/intup.xpm"
#include "images/intdown.xpm"
#include "images/icon.xpm"

static void on_Close_activate(GtkWidget *w, gpointer data)
{
	((DeviceWindow*)data)->hide();
}

static gint delete_event(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	((DeviceWindow*)data)->hide();
	return (TRUE); 
}

void DeviceWindow::tags()
{
	main->files->clear_tags();
	if (current_device != NULL)
//		current_device->tag = 1;
		current_device->selected = 1;
	if (current_interface != NULL)
		if (current_interface->network != NULL)
//			current_interface->network->tag = 1;
			current_interface->network->selected = 1;
	main->monitor->draw();
}

static void device_list_cb(GtkWidget *clist, gint row, gint column, GdkEventButton *event, gpointer data)
{
	DeviceWindow *m = (DeviceWindow*) data;
	gchar *text;

	gtk_clist_get_text(GTK_CLIST(clist), row, 0, &text);
	m->select_interface(text);
	m->tags();
}

static void device_list_cb2(GtkWidget *clist, gint row, gint column, GdkEventButton *event, gpointer data)
{
	DeviceWindow *m = (DeviceWindow*) data;
	m->current_device = NULL;
	m->current_interface = NULL;
	gtk_clist_clear(GTK_CLIST(m->clist4));
	gtk_clist_clear(GTK_CLIST(m->clist9));
	gtk_clist_set_column_title(GTK_CLIST(m->clist4), 0, "Interface");
	m->expose_device();
	m->tags();
}

static void iface_list_cb(GtkWidget *clist, gint row, gint column, GdkEventButton *event, gpointer data)
{
	DeviceWindow *m = (DeviceWindow*) data;
	gchar *text;

	gtk_clist_get_text(GTK_CLIST(clist), row, 1, &text);
	m->select_interface2(text);
	m->tags();
}

static void iface_list_cb2(GtkWidget *clist, gint row, gint column, GdkEventButton *event, gpointer data)
{
	DeviceWindow *m = (DeviceWindow*) data;
	m->current_interface = NULL;
	gtk_clist_clear(GTK_CLIST(m->clist9));
	m->expose_device();
	m->tags();
}

static void title_cb(GtkWidget *clist, gint column, gpointer data)
{
	gtk_clist_set_sort_column (GTK_CLIST (clist), column);
	gtk_clist_set_compare_func (GTK_CLIST (clist), NULL);
	gtk_clist_sort (GTK_CLIST (clist));
}

static gint expose_event(GtkWidget *w, GdkEventExpose *event, gpointer data)
{ ((DeviceWindow*)data)->expose_device(); return FALSE; }

DeviceWindow::DeviceWindow(Main *m)
{
	main = m;
	current_device = NULL;
	current_interface = NULL;
	create();
	gc = gdk_gc_new(device->window);
}

DeviceWindow::~DeviceWindow()
{
}

void DeviceWindow::create()
{
  device = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (device), "Device Details");
  gtk_window_set_policy (GTK_WINDOW (device), FALSE, FALSE, FALSE);
  gtk_widget_realize (device);
	window_icon(device, icon_xpm);
	gdk_window_set_icon_name(device->window, "Device");

	gtk_signal_connect(GTK_OBJECT(device), "delete_event",
		GTK_SIGNAL_FUNC(delete_event), this);

  table1 = gtk_table_new (2, 3, FALSE);
  gtk_widget_show (table1);
  gtk_container_add (GTK_CONTAINER (device), table1);
  gtk_container_set_border_width (GTK_CONTAINER (table1), 4);
  gtk_table_set_row_spacings (GTK_TABLE (table1), 4);
  gtk_table_set_col_spacings (GTK_TABLE (table1), 4);

  drawingarea3 = gtk_drawing_area_new ();
	gtk_widget_set_events(drawingarea3, GDK_EXPOSURE_MASK);
  gtk_widget_show (drawingarea3);
  gtk_table_attach (GTK_TABLE (table1), drawingarea3, 1, 3, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
  gtk_widget_set_usize (drawingarea3, 320, 100);
	gtk_widget_realize(drawingarea3);
	gtk_signal_connect (GTK_OBJECT (drawingarea3), "expose_event",
		(GtkSignalFunc) expose_event, this);

  scrolledwindow4 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow4);
  gtk_table_attach (GTK_TABLE (table1), scrolledwindow4, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
  gtk_widget_set_usize (scrolledwindow4, 160, -2);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow4), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  clist3 = gtk_clist_new (1);
  gtk_widget_show (clist3);
  gtk_container_add (GTK_CONTAINER (scrolledwindow4), clist3);
  gtk_clist_set_column_width (GTK_CLIST (clist3), 0, 80);
  gtk_clist_column_titles_show (GTK_CLIST (clist3));
	gtk_signal_connect(GTK_OBJECT(clist3), "select_row",
		GTK_SIGNAL_FUNC(device_list_cb), this);
	gtk_signal_connect(GTK_OBJECT(clist3), "unselect_row",
		GTK_SIGNAL_FUNC(device_list_cb2), this);

  label8 = gtk_label_new ("Device");
  gtk_widget_show (label8);
  gtk_clist_set_column_widget (GTK_CLIST (clist3), 0, label8);
  gtk_misc_set_alignment (GTK_MISC (label8), 0, 0.5);

  drawingarea4 = gtk_drawing_area_new ();
	gtk_widget_set_events(drawingarea4, GDK_EXPOSURE_MASK);
  gtk_widget_show (drawingarea4);
  gtk_table_attach (GTK_TABLE (table1), drawingarea4, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
  gtk_widget_set_usize (drawingarea4, 200, 100);
	gtk_widget_realize(drawingarea4);
	gtk_signal_connect (GTK_OBJECT (drawingarea4), "expose_event",
		(GtkSignalFunc) expose_event, this);

  scrolledwindow5 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow5);
  gtk_table_attach (GTK_TABLE (table1), scrolledwindow5, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
  gtk_widget_set_usize (scrolledwindow5, 160, -2);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow5), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  clist4 = gtk_clist_new (2);
  gtk_widget_show (clist4);
  gtk_container_add (GTK_CONTAINER (scrolledwindow5), clist4);
  gtk_clist_set_column_width (GTK_CLIST (clist4), 0, 16);
  gtk_clist_set_column_width (GTK_CLIST (clist4), 1, 80);
  gtk_clist_column_titles_show (GTK_CLIST (clist4));
	gtk_signal_connect(GTK_OBJECT(clist4), "select_row",
		GTK_SIGNAL_FUNC(iface_list_cb), this);
	gtk_signal_connect(GTK_OBJECT(clist4), "unselect_row",
		GTK_SIGNAL_FUNC(iface_list_cb2), this);

  label9 = gtk_label_new ("Interface");
  gtk_widget_show (label9);
  gtk_clist_set_column_widget (GTK_CLIST (clist4), 0, label9);
  gtk_misc_set_alignment (GTK_MISC (label9), 0, 0.5);

  vbox7 = gtk_vbox_new (FALSE, 4);
  gtk_widget_show (vbox7);
  gtk_table_attach (GTK_TABLE (table1), vbox7, 2, 3, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);

  scrolledwindow10 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow10);
  gtk_box_pack_start (GTK_BOX (vbox7), scrolledwindow10, TRUE, TRUE, 0);
  gtk_widget_set_usize (scrolledwindow10, 132, -2);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow10), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);

  clist9 = gtk_clist_new (1);
  gtk_widget_show (clist9);
  gtk_container_add (GTK_CONTAINER (scrolledwindow10), clist9);
  gtk_clist_set_column_width (GTK_CLIST (clist9), 0, 80);
  gtk_clist_column_titles_show (GTK_CLIST (clist9));

  label14 = gtk_label_new ("IP Addresses");
  gtk_widget_show (label14);
  gtk_clist_set_column_widget (GTK_CLIST (clist9), 0, label14);
  gtk_misc_set_alignment (GTK_MISC (label14), 0, 0.5);

  button3 = gtk_button_new_with_label ("Close");
  gtk_widget_show (button3);
  gtk_box_pack_start (GTK_BOX (vbox7), button3, FALSE, FALSE, 0);
  gtk_widget_set_usize (button3, 60, -2);

  gtk_signal_connect (GTK_OBJECT (button3), "clicked", GTK_SIGNAL_FUNC (on_Close_activate), this);
}

void DeviceWindow::show()
{
	init();
	gtk_widget_show(device);
	gdk_window_raise(device->window);
}

void DeviceWindow::hide()
{
	gtk_widget_hide(device);
}

void DeviceWindow::init()
{
	int row;
	Link *ln;
	Router *r;
	gchar *data[1];
	char buf[256];

	if (main->files->objects==NULL) return;

	gtk_clist_clear(GTK_CLIST(clist3));

	row = 0;
	for (ln=main->files->objects->routers.get_head(); ln!=NULL; ln=ln->next)
	{
		r = (Router*) ln->item;
		data[0] = strdup(r->name);
		gtk_clist_append(GTK_CLIST(clist3), data);
		delete data[0];
		gtk_clist_set_row_data(GTK_CLIST(clist3), row, r);
		row++;
	}

	sprintf(buf, "Device (%d)", row);
	gtk_clist_set_column_title(GTK_CLIST(clist3), 0, buf);

	title_cb(clist3, 0, this);
}

void DeviceWindow::select_interface(char *str)
{
	Link *ln, *ln2;
	Router *r;
	Interface *in;
	gchar *data[2];
	char buf[256];
	int row;
	GdkPixmap *gdkpixmap;
	GdkBitmap *mask;

	if (main->files->objects==NULL) return;

	for (ln=main->files->objects->routers.get_head(); ln!=NULL; ln=ln->next)
	{
		r = (Router*) ln->item;
		if ((strcmp(str, r->name)==0) && (current_device!=r))
		{
			current_device = r;
			current_interface = NULL;
			// update the interface list
			gtk_clist_freeze(GTK_CLIST(clist4));
			gtk_clist_clear(GTK_CLIST(clist4));
			row = 0;
			for (ln2=r->interfaces.get_head(); ln2!=NULL; ln2=ln2->next)
			{
				in = (Interface*) ln2->item;
				sprintf(buf, "%s", in->port);

				data[0] = strdup("");
				data[1] = strdup(buf);
				gtk_clist_append(GTK_CLIST(clist4), data);
				delete data[0]; delete data[1];

				if (in->status == 1)
					gdkpixmap = gdk_pixmap_create_from_xpm_d(device->window, &mask,
						&device->style->bg[GTK_STATE_NORMAL], (gchar **)intup_xpm);
				else
					gdkpixmap = gdk_pixmap_create_from_xpm_d(device->window, &mask,
						&device->style->bg[GTK_STATE_NORMAL], (gchar **)intdown_xpm);
				gtk_clist_set_pixmap(GTK_CLIST(clist4), row, 0, gdkpixmap, mask);
				gdk_pixmap_unref(gdkpixmap);
				gdk_bitmap_unref(mask);

				row++;
			}
			sprintf(buf, "Interface (%d)", row);
			gtk_clist_set_column_title(GTK_CLIST(clist4), 0, buf);
			gtk_clist_thaw(GTK_CLIST(clist4));
		}
	}
	expose_device();
}

void DeviceWindow::select_interface2(char *str)
{
	Interface *in;
	Link *ln, *ln2;
	char buf[256];
	IPAddress *ip;
	gchar *data[1];

	for (ln=current_device->interfaces.get_head(); ln!=NULL; ln=ln->next)
	{
		in = (Interface*) ln->item;
		sprintf(buf, "%s", in->port);
		if ((strcmp(str, buf)==0) && (current_interface!=in))
		{
			current_interface = in;
			gtk_clist_clear(GTK_CLIST(clist9));
			for (ln2=in->ip_addresses.get_head(); ln2!=NULL; ln2=ln2->next)
			{
				ip = (IPAddress*) ln2->item;
				ip->sprint(buf);
				data[0] = strdup(buf);
				gtk_clist_append(GTK_CLIST(clist9), data);
				delete data[0];
			}
		}
	}
	expose_device();
}

void DeviceWindow::expose_device()
{
	int x, y, w, h;
	GtkStyle *style;
	char buf[256];

	gdk_draw_rectangle(drawingarea4->window, drawingarea4->style->bg_gc[GTK_STATE_NORMAL], TRUE, 0, 0, drawingarea4->allocation.width, drawingarea4->allocation.height);
	gdk_draw_rectangle(drawingarea3->window, drawingarea3->style->bg_gc[GTK_STATE_NORMAL], TRUE, 0, 0, drawingarea3->allocation.width, drawingarea3->allocation.height);

	x = drawingarea4->allocation.width;
	y = drawingarea4->allocation.height;
	w = x/128;
	h = y/128;
	style = gtk_widget_get_style(device);
	for (x=0; x<=w; x++)
		for (y=0; y<=h; y++)
			gtk_draw_flat_box(style, drawingarea4->window, GTK_STATE_NORMAL,
				GTK_SHADOW_NONE, x*128, y*128, 128, 128);

	x = drawingarea3->allocation.width;
	y = drawingarea3->allocation.height;
	w = x/128;
	h = y/128;
	style = gtk_widget_get_style(device);
	for (x=0; x<=w; x++)
		for (y=0; y<=h; y++)
			gtk_draw_flat_box(style, drawingarea3->window, GTK_STATE_NORMAL,
				GTK_SHADOW_NONE, x*128, y*128, 128, 128);

	if (current_device!=NULL)
	{
		gdk_gc_set_foreground(gc, &main->black);
		gdk_draw_text(drawingarea3->window, main->font12b, gc, 2, 12,
		"NAME", 4);
		gdk_draw_text(drawingarea3->window, main->font12m, gc, 102, 12,
			current_device->name, strlen(current_device->name));

		gdk_draw_text(drawingarea3->window, main->font12b, gc, 2, 24,
		"DESCRIPTION", 11);
		gdk_draw_text(drawingarea3->window, main->font12m, gc, 102, 24,
			current_device->descr, strlen(current_device->descr));

		gdk_draw_text(drawingarea3->window, main->font12b, gc, 2, 36,
		"LOCATION", 8);
		gdk_draw_text(drawingarea3->window, main->font12m, gc, 102, 36,
			current_device->location, strlen(current_device->location));

		gdk_draw_text(drawingarea3->window, main->font12b, gc, 2, 48,
		"CONTACT", 7);
		gdk_draw_text(drawingarea3->window, main->font12m, gc, 102, 48,
			current_device->contact, strlen(current_device->contact));

		gdk_draw_text(drawingarea3->window, main->font12b, gc, 2, 60,
			"COMMUNITY", 9);
		gdk_draw_text(drawingarea3->window, main->font12m, gc, 102, 60,
			current_device->community, strlen(current_device->community));

		gdk_draw_text(drawingarea3->window, main->font12b, gc, 2, 72,
			"LAST SEEN", 9);
		date_to_str(gmtime(&current_device->last_seen), buf, 1);
		gdk_draw_text(drawingarea3->window, main->font12m, gc, 102, 72,
			buf, strlen(buf));

		gdk_draw_text(drawingarea3->window, main->font12b, gc, 2, 84,
			"REGISTERED", 10);
		if (current_device->registered == 0)
			strcpy(buf, "no"); else strcpy(buf, "yes");
		gdk_draw_text(drawingarea3->window, main->font12m, gc, 102, 84,
			buf, strlen(buf));

		gdk_draw_text(drawingarea3->window, main->font12b, gc, 2, 96,
			"STATUS", 6);
		if (current_device->status == 0)
			strcpy(buf, "down"); else strcpy(buf, "up");
		gdk_draw_text(drawingarea3->window, main->font12m, gc, 102, 96,
			buf, strlen(buf));
	}

	if (current_interface!=NULL)
	{
		gdk_gc_set_foreground(gc, &main->black);
		gdk_draw_text(drawingarea4->window, main->font12b, gc, 2, 12,
			"INDEX", 5);
		sprintf(buf, "%d", current_interface->index);
		gdk_draw_text(drawingarea4->window, main->font12m, gc, 102, 12,
			buf, strlen(buf));

		gdk_draw_text(drawingarea4->window, main->font12b, gc, 2, 24,
			"DESCRIPTION", 11);
		gdk_draw_text(drawingarea4->window, main->font12m, gc, 102, 24,
			current_interface->descr, strlen(current_interface->descr));

		gdk_draw_text(drawingarea4->window, main->font12b, gc, 2, 36,
			"PORT", 4);
		gdk_draw_text(drawingarea4->window, main->font12m, gc, 102, 36,
			current_interface->port, strlen(current_interface->port));

		gdk_draw_text(drawingarea4->window, main->font12b, gc, 2, 48,
			"MAC ADDRESS", 11);
		sprint_mac(current_interface->mac, buf);
		gdk_draw_text(drawingarea4->window, main->font12m, gc, 102, 48,
			buf, strlen(buf));

		gdk_draw_text(drawingarea4->window, main->font12b, gc, 2, 60,
			"SPEED", 5);
		speed_to_str(current_interface->speed, buf);
		gdk_draw_text(drawingarea4->window, main->font12m, gc, 102, 60,
			buf, strlen(buf));

		gdk_draw_text(drawingarea4->window, main->font12b, gc, 2, 72,
			"STATUS", 6);
		if (current_interface->status == 1)
			strcpy(buf, "up"); else strcpy(buf, "down");
		gdk_draw_text(drawingarea4->window, main->font12m, gc, 102, 72,
			buf, strlen(buf));

		gdk_draw_text(drawingarea4->window, main->font12b, gc, 2, 84,
			"TYPE", 4);
		interface_type_to_str(current_interface->type, buf);
		gdk_draw_text(drawingarea4->window, main->font12m, gc, 102, 84,
			buf, strlen(buf));
	}
}

void DeviceWindow::device_string(char *str)
{
	gchar *text;
	int i;

	if (main->files->objects==NULL) return;

	for (i=0; i<main->files->objects->routers.size(); i++)
	{
		gtk_clist_get_text(GTK_CLIST(clist3), i, 0, &text);
		if (strcmp(text, str)==0)
		{
			gtk_clist_moveto(GTK_CLIST(clist3), i, 0, 0.5, 0);
			gtk_clist_select_row(GTK_CLIST(clist3), i, 0);
		}
	}
}

