#include <gtk/gtk.h>
#include <stdlib.h>
#include "main.h"
#include "diagram.h"
#include "machine.h"
#include "network.h"
#include "router.h"
#include "subnet.h"
#include "interface.h"
#include "misc.h"

Diagram::Diagram(Main *m)
{
	main = m;
	points = new GdkPoint[4];
	animate_objects = NULL;
	labels = 0;
}

Diagram::~Diagram()
{
	delete points;
}

// w, h are the dimensions of the canvas to draw into
// ox, oy are scrollbar values x ,y for top left after zoomed
// zoom is the zoom factor used

void Diagram::dodraw(GdkPixmap *cnvs, GdkGC *gc, double zoom, int w, int h, int ox, int oy, int g)
{
	Machine *m;
	Network *n;
	Router *r;
	Subnet *sn;
	Interface *in;
	gint i, countm, counts, show_font;
	gint ww;
	Link *ln, *ln2;
	char nm[256], tmpnm[256];
	GdkFont *fnt;
	double zoomfactor;
	gint8 dash_list[2];

	if (cnvs==NULL) return;

	if (zoom<0.0)   zoom = 0.0;
	if (zoom>100.0) zoom = 100.0;

	if (main->pref->auto_move_links) network_links();

	gdk_gc_set_line_attributes (gc, 1+((int)(zoom-0.5)),
		GDK_LINE_SOLID, GDK_CAP_NOT_LAST, GDK_JOIN_MITER);

	// work out which font to use
	show_font = 1;
	zoomfactor = 16*zoom;
	if (zoomfactor<8) { show_font = 0; } else
	if (zoomfactor<12) { fnt = main->font8b; } else
	if (zoomfactor<18) { fnt = main->font12b; } else
	if (zoomfactor<24) { fnt = main->font18b; } else
	{ fnt = main->font24b; }

	if (g == 1)
	{
		// clear window
		gdk_gc_set_foreground(gc, &main->white);
		gdk_draw_rectangle(cnvs, gc, TRUE, 0, 0, w, h);

		// grid
		if (main->monitor->grid > 0)
		{
			gdk_gc_set_foreground(gc, &main->lightgray);
			for(i=int(main->monitor->grid*zoom); i<h;
					i+=int(main->monitor->grid*zoom))
				gdk_draw_line(cnvs, gc, 0, i, w, i);
			for(i=int(main->monitor->grid*zoom); i<w;
					i+=int(main->monitor->grid*zoom))
				gdk_draw_line(cnvs, gc, i, 0, i, h);
		}
	}

	if (main->files->objects==NULL) return;

	// draw connecting lines

	gdk_gc_set_foreground(gc, &main->blue);
	if (main->monitor->ants==1)
		gdk_gc_set_line_attributes (gc, 1+((int)(zoom-0.5)),
			GDK_LINE_ON_OFF_DASH, GDK_CAP_NOT_LAST, GDK_JOIN_MITER);
	for (ln=main->files->objects->networks.get_head(); ln!=NULL; ln=ln->next)
	{
		n = (Network*) ln->item;
		if (n->show==1)
		{
			for (ln2=n->interfaces.get_head(); ln2!=NULL; ln2=ln2->next)
			{
				in = (Interface*) ln2->item;
				r = in->router;
				interface_color(gc, in->type);
				if (main->monitor->ants==1)
				{
					dash_list[0] = 6;
					dash_list[1] = 2;
					gdk_gc_set_dashes(gc, main->monitor->animate_frame,
						dash_list, sizeof(dash_list));
				}
				gdk_draw_line(cnvs, gc, gint(((n->x+(n->w/2))*zoom)-ox),
					gint(((n->y+(n->h/2))*zoom)-oy), gint(((r->x+(r->w/2))*zoom)-ox),
					gint(((r->y+(r->w/2))*zoom)-oy));
			}
		}
	}
	if (main->monitor->ants==1)
		gdk_gc_set_line_attributes (gc, 1+((int)(zoom-0.5)),
			GDK_LINE_SOLID, GDK_CAP_NOT_LAST, GDK_JOIN_MITER);

	// draw the clouds

	for (ln=main->files->objects->networks.get_head(); ln!=NULL; ln=ln->next)
	{
		n = (Network*) ln->item;
		if (n->show==1)
		{
			if (n->view == 0)
			{
				// draw little circle
				gdk_gc_set_foreground(gc, &main->blue);
				if ((n->selected == 1) && (g == 1))
					gdk_gc_set_foreground(gc, &main->darkblue);
				if (n->machines.size()==0)
				{
					gdk_gc_set_foreground(gc, &main->cyan);
					if ((n->selected == 1) && (g == 1))
						gdk_gc_set_foreground(gc, &main->darkcyan);
				}
				gdk_draw_arc(cnvs, gc, TRUE,
					gint(((n->x)*zoom)-ox), gint(((n->y)*zoom)-oy),
					gint((n->w)*zoom), gint((n->h)*zoom), 0, 360*64);
			}else
			if (n->view == 1)
			{
				counts = n->subnets.size();
				if (counts==0)
				{
					gdk_gc_set_foreground(gc, &main->blue);
					if (n->selected == 1) gdk_gc_set_foreground(gc, &main->darkblue);
					gdk_draw_arc(cnvs, gc, TRUE,
						gint(((n->x)*zoom)-ox), gint(((n->y)*zoom)-oy),
						gint((n->w)*zoom), gint((n->h)*zoom), 0, 360*64);
				}else
				{
					n->w = 100;
					n->h = (counts*main->font10b->ascent)+4;
					gdk_gc_set_foreground(gc, &main->white);
					gdk_draw_rectangle (cnvs, gc, TRUE, gint(((n->x)*zoom)-ox),
						gint(((n->y)*zoom)-oy), gint((n->w)*zoom), gint((n->h)*zoom));
					gdk_gc_set_foreground(gc, &main->blue);
					gdk_draw_rectangle (cnvs, gc, FALSE, gint(((n->x)*zoom)-ox),
						gint(((n->y)*zoom)-oy), gint((n->w)*zoom), gint((n->h)*zoom));
					if (g == 1)
					{
						i = 1;
						for (ln2=n->subnets.get_head(); ln2!=NULL; ln2=ln2->next)
						{
							sn = (Subnet*) ln2->item;
							if (strlen(sn->name)>0) strcpy(nm, sn->name);
							else
							{
								sn->ip.sprint(tmpnm);
								sprintf(nm, "%s / %d", tmpnm, sn->mask);
							}
							gdk_gc_set_foreground(gc, &main->black);
							gdk_draw_text(cnvs, main->font10b, gc,
								gint((n->x)*zoom)+2-ox,
								gint((n->y)*zoom)+2+(main->font10b->ascent*i)-oy,
								nm, strlen(nm));
							i++;
						}
					}
				}
			}else
			{
				countm = n->machines.size();
				n->w = 100;
				n->h = (countm*main->font10b->ascent)+4;
				gdk_gc_set_foreground(gc, &main->white);
				gdk_draw_rectangle (cnvs, gc, TRUE, gint(((n->x)*zoom)-ox),
					gint(((n->y)*zoom)-oy), gint((n->w)*zoom), gint((n->h)*zoom));
				gdk_gc_set_foreground(gc, &main->blue);
				gdk_draw_rectangle (cnvs, gc, FALSE, gint(((n->x)*zoom)-ox),
					gint(((n->y)*zoom)-oy), gint((n->w)*zoom), gint((n->h)*zoom));
				if (g == 1)
				{
					i = 1;
					for (ln2 = n->machines.get_head(); ln2!=NULL; ln2=ln2->next)
					{
						m = (Machine*) ln2->item;
						if (strlen(m->name)>0) strcpy(nm, m->name); else
						if (m->ip.zero()==0) m->ip.sprint(nm); else
							sprint_mac(m->mac, nm);

						gdk_gc_set_foreground(gc, &main->black);
						gdk_draw_text(cnvs, main->font10b, gc,
							gint((n->x)*zoom)+2-ox,
							gint((n->y)*zoom)+2+(main->font10b->ascent*i)-oy,
							nm, strlen(nm));
						i++;
					}
				}
			}

			if (n->selected == 1)
			{
				gdk_gc_set_foreground(gc, &main->black);
				gdk_draw_arc(cnvs, gc, FALSE,
					gint(((n->x)*zoom)-ox), gint(((n->y)*zoom)-oy),
					gint((n->w)*zoom), gint((n->h)*zoom), 0, 360*64);
			}

		}
	}

// devices

	for (ln=main->files->objects->routers.get_head(); ln!=NULL; ln=ln->next)
	{
		r = (Router*) ln->item;
		if (r->show == 1)
		{
			if (r->status == 1)
			{
				gdk_gc_set_foreground(gc, &main->green);
				if ((r->selected == 1) && (g == 1))
					gdk_gc_set_foreground(gc, &main->darkgreen);
			}else
			{
				gdk_gc_set_foreground(gc, &main->red);
				if ((r->selected == 1) && (g == 1))
					gdk_gc_set_foreground(gc, &main->darkred);
			}
			if (r->type == 's')
			{
				gdk_draw_arc(cnvs, gc, TRUE,
					gint(((r->x)*zoom)-ox), gint(((r->y)*zoom)-oy),
					gint((r->w)*zoom), gint((r->h)*zoom), 0, 360*64);
			}else
			if (r->type == 'r')
			{
				points[0].x = gint(((r->x+(r->w/2))*zoom)-ox);
				points[0].y = gint(((r->y)*zoom)-oy);
				points[1].x = gint(((r->x+r->w)*zoom)-ox);
				points[1].y = gint(((r->y+(r->h/2))*zoom)-oy);
				points[2].x = gint(((r->x+(r->w/2))*zoom)-ox);
				points[2].y = gint(((r->y+r->h)*zoom)-oy);
				points[3].x = gint(((r->x)*zoom)-ox);
				points[3].y = gint(((r->y+(r->h/2))*zoom)-oy);
				gdk_draw_polygon(cnvs, gc, TRUE, points, 4);
			}else
			{
				points[0].x = gint(((r->x)*zoom)-ox);
				points[0].y = gint(((r->y+r->h)*zoom)-oy);
				points[1].x = gint(((r->x+(r->w/2))*zoom)-ox);
				points[1].y = gint(((r->y)*zoom)-oy);
				points[2].x = gint(((r->x+r->w)*zoom)-ox);
				points[2].y = gint(((r->y+r->h)*zoom)-oy);
				gdk_draw_polygon(cnvs, gc, TRUE, points, 3);
			}

			if ((r->selected == 1) && (g == 1))
			{
				gdk_gc_set_foreground(gc, &main->black);
				if (r->type == 's')
				{
					gdk_draw_arc(cnvs, gc, FALSE,
						gint(((r->x)*zoom)-ox), gint(((r->y)*zoom)-oy),
						gint((r->w)*zoom), gint((r->h)*zoom), 0, 360*64);
				}else
				if (r->type == 'r')
				{
					points[0].x = gint(((r->x+(r->w/2))*zoom)-ox);
					points[0].y = gint(((r->y)*zoom)-oy);
					points[1].x = gint(((r->x+r->w)*zoom)-ox);
					points[1].y = gint(((r->y+(r->h/2))*zoom)-oy);
					points[2].x = gint(((r->x+(r->w/2))*zoom)-ox);
					points[2].y = gint(((r->y+r->h)*zoom)-oy);
					points[3].x = gint(((r->x)*zoom)-ox);
					points[3].y = gint(((r->y+(r->h/2))*zoom)-oy);
					gdk_draw_polygon(cnvs, gc, FALSE, points, 4);
				}else
				{
					points[0].x = gint(((r->x)*zoom)-ox);
					points[0].y = gint(((r->y+r->h)*zoom)-oy);
					points[1].x = gint(((r->x+(r->w/2))*zoom)-ox);
					points[1].y = gint(((r->y)*zoom)-oy);
					points[2].x = gint(((r->x+r->w)*zoom)-ox);
					points[2].y = gint(((r->y+r->h)*zoom)-oy);
					gdk_draw_polygon(cnvs, gc, FALSE, points, 3);
				}
			}

			if ((main->monitor->diagram->labels == 1) && (show_font == 1))
			{
				ww = gdk_text_width(fnt, r->name, strlen(r->name));
				gdk_gc_set_foreground(gc, &main->white);
				gdk_draw_text(cnvs, fnt, gc,
					gint((r->x)*zoom)+int(8*zoom)-(ww/2)-ox-1,
					gint((r->y)*zoom)+int(16*zoom)+fnt->ascent+4-oy-1,
					r->name, strlen(r->name));
				gdk_draw_text(cnvs, fnt, gc,
					gint((r->x)*zoom)+int(8*zoom)-(ww/2)-ox+1,
					gint((r->y)*zoom)+int(16*zoom)+fnt->ascent+4-oy-1,
					r->name, strlen(r->name));
				gdk_draw_text(cnvs, fnt, gc,
					gint((r->x)*zoom)+int(8*zoom)-(ww/2)-ox+1,
					gint((r->y)*zoom)+int(16*zoom)+fnt->ascent+4-oy+1,
					r->name, strlen(r->name));
				gdk_draw_text(cnvs, fnt, gc,
					gint((r->x)*zoom)+int(8*zoom)-(ww/2)-ox-1,
					gint((r->y)*zoom)+int(16*zoom)+fnt->ascent+4-oy+1,
					r->name, strlen(r->name));
				gdk_gc_set_foreground(gc, &main->black);
				gdk_draw_text(cnvs, fnt, gc,
					gint((r->x)*zoom)+int(8*zoom)-(ww/2)-ox,
					gint((r->y)*zoom)+int(16*zoom)+fnt->ascent+4-oy,
					r->name, strlen(r->name));
			}
		}
	}
}

void Diagram::network_links()
{
	Link *ln, *ln2;
	Network *n;
	Router *r1, *r2;
	Interface *in;

	for (ln=main->files->objects->networks.get_head(); ln!=NULL; ln=ln->next)
	{
		n = (Network*) ln->item;
		if (n->interfaces.size() == 2)
		{
			ln2 = n->interfaces.get_head();
			in = (Interface*) ln2->item;
			r1 = in->router;
			ln2 = ln2->next;
			in = (Interface*) ln2->item;
			r2 = in->router;

			n->x = r1->x + ((r2->x - r1->x)/2) + 4;	// (width_router-width_network)/2
			n->y = r1->y + ((r2->y - r1->y)/2) + 4;	// is 4
		}
	}
}

void Diagram::interface_color(GdkGC *gc, int type)
{
	if (main->monitor->colours == 0)
	{
		gdk_gc_set_foreground(gc, &main->black);
	}else
	switch (type)
	{
		case 6:  gdk_gc_set_foreground(gc, &main->green); break;
		case 15: gdk_gc_set_foreground(gc, &main->cyan); break;
		case 18:
		case 22:
		case 23:
		case 28: gdk_gc_set_foreground(gc, &main->red); break;
		case 37:
		case 39:
		case 49:
		case 53:
		case 59: gdk_gc_set_foreground(gc, &main->yellow); break;
		case 1:
		case 24:
		default: gdk_gc_set_foreground(gc, &main->blue);
	}
}

void Diagram::autoplace_init()
{
	Link *ln, *ln2, *ln3;
	Edge *e;
	Node *nd;
	Network *n;
	Interface *in;
	Node *n1;
	Visual *v, *v2;
	int countr, counts;
	Router *r;

	if (main->files->objects==NULL) return;

// delete the old lists here
	for (ln=nodes.get_head(); ln!=NULL; ln=ln->next)
	{
		nd = (Node*) ln->item;
		delete nd;
	}
	nodes.clear();
	for (ln=edges.get_head(); ln!=NULL; ln=ln->next)
	{
		e = (Edge*) ln->item;
		delete e;
	}
	edges.clear();

// start by building a list of nodes

	for (ln=main->files->objects->networks.get_head(); ln!=NULL; ln=ln->next)
	{
		v = (Visual*) ln->item;
		v->fix = 0;
		if (v->show==1)
		{
			nd = new Node();
			nd->dx = 0;
			nd->dy = 0;
			nd->obj = v;
			nodes.add(nd);
		}
	}
	for (ln=main->files->objects->routers.get_head(); ln!=NULL; ln=ln->next)
	{
		v = (Visual*) ln->item;
		v->fix = 0;
		if ((v->show == 1))
		{
			nd = new Node();
			nd->dx = 0;
			nd->dy = 0;
			nd->obj = v;
			nodes.add(nd);
		}
	}

// now build a list of edges

	for (ln=nodes.get_head(); ln!=NULL; ln=ln->next)
	{
		nd = (Node*) ln->item;
		v = (Visual*) nd->obj;
		if (v->type == 'n')
		{
			n = (Network*) v;
			if (n->show==1)		// for each network node
			{
				countr = 0;
				counts = 0;
				for (ln2=n->interfaces.get_head(); ln2!=NULL; ln2=ln2->next)
				{
					in = (Interface*) ln2->item;
					r = in->router;
					if (r->type == 'r') countr++;
					if (r->type == 's') counts++;
				}

// first count the number of networks attached to only one device
// set edges for these networks
// set count in devices if current count = 0
// set count in networks if current count = 0
// goto line 3 until all counts are set

/*
				if (countr+counts==1)
				{
				}
*/

				// this one works
				for (ln2=n->interfaces.get_head(); ln2!=NULL; ln2=ln2->next)
				{
					in = (Interface*) ln2->item;
					r = in->router;
					for (ln3=nodes.get_head(); ln3!=NULL; ln3=ln3->next)
					{
						n1 = (Node*) ln3->item;
						v2 = (Visual*) n1->obj;
						if (v2 == r)
						{
							e = new Edge();
							e->from = n1;
							e->to = nd;
							e->len = 25+(countr*10)+(counts*5);
							edges.add(e);
						}
					}
				}
			}
		}
	}
	animate_objects = main->files->objects;
}

Node *Diagram::get_node(Visual*v)
{
	Link *ln;
	Node *nd;

	for (ln=nodes.get_head(); ln!=NULL; ln=ln->next)
	{
		nd = (Node*) ln->item;
		if (nd->obj==v)
			return nd;
	}
	return NULL;
}

void Diagram::autoplace()
{
	Link *ln, *ln2;
	Edge *e;
	Node *n1, *n2;
	double vx, vy, dlen, len, f, dx, dy;

	if (animate_objects != main->files->objects)
		autoplace_init();

//	printf("AUTO 1\n");
	for (ln=edges.get_head(); ln!=NULL; ln=ln->next)
	{
		e = (Edge*) ln->item;
		vx = e->to->obj->x - e->from->obj->x;
		vy = e->to->obj->y - e->from->obj->y;
//		len = sqrt(vx*vx + vy*vy);
		len = sqrt(vx*vx + vy*vy);
		if (len<5) len=5;
		f = (e->len - len) / (len*3);
		dx = f * vx;
		dy = f * vy;

		e->to->dx += dx;
		e->to->dy += dy;
		e->from->dx += -dx;
		e->from->dy += -dy;
	}

//	printf("AUTO 2\n");
	for (ln=nodes.get_head(); ln!=NULL; ln=ln->next)
	{
		n1 = (Node*) ln->item;
		dx = 0;
		dy = 0;
		while(gtk_events_pending()) gtk_main_iteration();
		for (ln2=nodes.get_head(); ln2!=NULL; ln2=ln2->next)
		{
			n2 = (Node*) ln2->item;
			if (n1!=n2)
			{
				vx = n1->obj->x - n2->obj->x;
				vy = n1->obj->y - n2->obj->y;
				len = vx*vx + vy*vy;
				if (len == 0)
				{
					dx += rand()%10;
					dy += rand()%10;
				}else
				if (len < 100*100)
				{
					dx += vx / len;
					dy += vy / len;
				}
			}
		}
		dlen = dx*dx + dy*dy;
		if (dlen>0)
		{
			dlen = sqrt(dlen) / 2;
			n1->dx += dx / dlen;
			n1->dy += dy / dlen;
		}
	}

//	printf("AUTO 3\n");
	for (ln=nodes.get_head(); ln!=NULL; ln=ln->next)
	{
		n1 = (Node*) ln->item;
		if (n1->obj->fix == 0)
		{
			if (n1->dx<5) vx=n1->dx; else vx=5;
			if (vx<-5) vx=-5;
			n1->obj->x += (int) vx;
			if (n1->dy<5) vy=n1->dy; else vy=5;
			if (vy<-5) vy=-5;
			n1->obj->y += (int) vy;
			n1->dx /= 1.5;
			n1->dy /= 1.5;
		}
	}
//	printf("DRAW\n");
	main->monitor->draw();
}

void Diagram::compress(int sel)
{
	Link *ln;
	Router *r;
	Network *n;
	int i;
	double minx, miny, maxx, maxy;

	i = 0;
	minx = miny = maxx = maxy = 0;
	full_height = (int) (maxy-miny+20);
	full_width = (int) (maxx-minx+16);
	if (main->files->objects==NULL) return;

	for (ln=main->files->objects->routers.get_head(); ln!=NULL; ln=ln->next)
	{
		r = (Router*) ln->item;
		if ((r->show) && ((sel==0) || (r->selected)))
		{
			if (i==0)
			{	minx = r->x; maxx = r->x+r->w; miny = r->y; maxy = r->y+r->h; i=1; }
			else
			{
				if (minx > r->x) minx = r->x;
				if (maxx < r->x+r->w) maxx = r->x+r->w;
				if (miny > r->y) miny = r->y;
				if (maxy < r->y+r->h) maxy = r->y+r->h;
			}
		}
	}
	for (ln=main->files->objects->networks.get_head(); ln!=NULL; ln=ln->next)
	{
		n = (Network*) ln->item;
		if ((n->show) && ((sel==0) || (n->selected)))
		{
			if (i==0)
			{	minx = n->x; maxx = n->x+n->w; miny = n->y; maxy = n->y+n->h; i=1; }
			else
			{
				if (minx > n->x) minx = n->x;
				if (maxx < n->x+n->w) maxx = n->x+n->w;
				if (miny > n->y) miny = n->y;
				if (maxy < n->y+n->h) maxy = n->y+n->h;
			}
		}
	}
	for (ln=main->files->objects->routers.get_head(); ln!=NULL; ln=ln->next)
	{
		r = (Router*) ln->item;
		r->x-=minx-8; r->y-=miny-8;
	}
	for (ln=main->files->objects->networks.get_head(); ln!=NULL; ln=ln->next)
	{
		n = (Network*) ln->item;
		n->x-=minx-8; n->y-=miny-8;
	}

	full_height = (int) (maxy-miny+20);
	full_width = (int) (maxx-minx+16);
}

