/*****************************************************************************
 Copyright  Newcastle University
 Author      Paul Coates
 *****************************************************************************/
#include <gtk/gtk.h>
#include <unistd.h>
#include <stdio.h>
#include <pwd.h>
#include "main.h"

int main(int argc, char *argv[])
{
	Main m;

	gtk_init(&argc, &argv);
	m.run();
	gtk_main();
}

gint timeout_callback(gpointer data)
{
	((Main *)data)->timeout();
	return TRUE;
}

gint init_timeout_callback(gpointer data)
{
	((Main *)data)->init_timeout();
	return FALSE;
}

Main::Main()
{
}

Main::~Main()
{
	delete files;
	delete monitor;
	delete help;
	delete filesel;
	delete search;
	delete pref;
	delete machine;
	delete subnet;
	delete device;
	delete bounds;
	delete filter;
}

void Main::alloc_color(GdkColor &col, guint16 r, guint16 g, guint16 b)
{
	col.red   = r;
	col.green = g;
	col.blue  = b;
	gdk_color_alloc(colormap, &col);
}

void Main::run()
{
	monitor = new Monitor(this);
	help = new Help(this);
	print = new Print(this);
	filesel = new FileSel(this);
	files = new Files(this);
	search = new Search(this);
	pref = new Pref(this);
	machine = new MachineWindow(this);
	subnet = new SubnetWindow(this);
	device = new DeviceWindow(this);
	bounds = new Bounds(this);
	filter = new FilterWindow(this);

	colormap = gdk_colormap_get_system();

	alloc_color(red, 65535, 0, 0);
	alloc_color(green, 0, 65535, 0);
	alloc_color(blue, 0, 0, 65535);

	alloc_color(darkred, 32767, 0, 0);
	alloc_color(darkgreen, 0, 32767, 0);
	alloc_color(darkblue, 0, 0, 32767);

	alloc_color(yellow, 65535, 65535, 0);
	alloc_color(cyan, 0, 65535, 65535);
	alloc_color(magenta, 65535, 0, 65535);

	alloc_color(darkyellow, 32767, 32767, 0);
	alloc_color(darkcyan, 0, 32767, 32767);
	alloc_color(darkmagenta, 32767, 0, 32767);

	alloc_color(lightgray, 49151, 49151, 49151);
	alloc_color(gray, 32767, 32767, 32767);
	alloc_color(darkgray, 16383, 16383, 16383);

	alloc_color(black, 0, 0, 0);
	alloc_color(white, 65535, 65535, 65535);
	alloc_color(orange, 65535, 32767, 32767);
	alloc_color(purple, 32767, 0, 65535);

	font8m  = gdk_font_load("-*-helvetica-medium-r-*-*-8-*-*-*-*-*-*-*");
	font10m = gdk_font_load("-*-helvetica-medium-r-*-*-10-*-*-*-*-*-*-*");
	font12m = gdk_font_load("-*-helvetica-medium-r-*-*-12-*-*-*-*-*-*-*");
	font18m = gdk_font_load("-*-helvetica-medium-r-*-*-18-*-*-*-*-*-*-*");
	font24m = gdk_font_load("-*-helvetica-medium-r-*-*-24-*-*-*-*-*-*-*");
	font8b  = gdk_font_load("-*-helvetica-bold-r-*-*-8-*-*-*-*-*-*-*");
	font10b = gdk_font_load("-*-helvetica-bold-r-*-*-10-*-*-*-*-*-*-*");
	font12b = gdk_font_load("-*-helvetica-bold-r-*-*-12-*-*-*-*-*-*-*");
	font18b = gdk_font_load("-*-helvetica-bold-r-*-*-18-*-*-*-*-*-*-*");
	font24b = gdk_font_load("-*-helvetica-bold-r-*-*-24-*-*-*-*-*-*-*");

	gtk_timeout_add(10000, timeout_callback, this);
	gtk_timeout_add(0, init_timeout_callback, this);
}

void Main::init_timeout()
{
	pref->load();

	if (files->check())
	{
		files->load_view(files->current_view);
//		pref->show();
//		bounds->show();
//		machine->show();
//		subnet->show();
//		device->show();
		monitor->show();
		monitor->draw();
	}
}

void Main::timeout()
{
	if (files->check())
	{
		monitor->draw();
	}
}

void Main::quit()
{
	// save code here
	gtk_exit(0);
}

