/*****************************************************************************
 Copyright  Newcastle University
 Author      Paul Coates
 *****************************************************************************/
#include <gtk/gtk.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <pwd.h>
#include "main.h"
#include "misc.h"
#include "images/icon.xpm"

static void on_Close_activate(GtkWidget *w, gpointer data)
{
	((Pref*)data)->hide();
}

static void on_Apply_activate(GtkWidget *w, gpointer data)
{
	((Pref*)data)->copy_from_widgets();
	((Pref*)data)->save();
	((Pref*)data)->hide();
}

static gint delete_event(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	((Pref*)data)->hide();
	return (TRUE); 
}

Pref::Pref(Main *m)
{
	struct passwd *p;

	main = m;
	// default settings
	p = getpwuid(getuid());

	sprintf(server_directory, "%s/%s", p->pw_dir, HOME_DIR);
	auto_bunch = 0;
	save_view_on_quit = 0;
	auto_move_links = 0;
	show_grid = 0;
	create();
	load();
}

Pref::~Pref()
{
}

void Pref::create()
{
  preference = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (preference), "Settings");
  gtk_window_set_policy (GTK_WINDOW (preference), FALSE, FALSE, FALSE);
  gtk_widget_realize (preference);
	window_icon(preference, icon_xpm);
	gdk_window_set_icon_name(preference->window, "Settings");

	gtk_signal_connect(GTK_OBJECT(preference), "delete_event",
		GTK_SIGNAL_FUNC(delete_event), this);

  vbox11 = gtk_vbox_new (FALSE, 4);
  gtk_widget_show (vbox11);
  gtk_container_add (GTK_CONTAINER (preference), vbox11);
  gtk_container_set_border_width (GTK_CONTAINER (vbox11), 4);

  notebook2 = gtk_notebook_new ();
  gtk_widget_show (notebook2);
  gtk_box_pack_start (GTK_BOX (vbox11), notebook2, FALSE, TRUE, 0);

  vbox33 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox33);
  gtk_container_add (GTK_CONTAINER (notebook2), vbox33);

  hbox5 = gtk_hbox_new (FALSE, 4);
  gtk_widget_show (hbox5);
  gtk_box_pack_start (GTK_BOX (vbox33), hbox5, FALSE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (hbox5), 4);

  label24 = gtk_label_new ("Server Directory");
  gtk_widget_show (label24);
  gtk_box_pack_start (GTK_BOX (hbox5), label24, FALSE, FALSE, 0);

  entry1 = gtk_entry_new ();
  gtk_widget_show (entry1);
  gtk_box_pack_start (GTK_BOX (hbox5), entry1, TRUE, TRUE, 0);

  label44 = gtk_label_new ("Server");
  gtk_widget_show (label44);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook2),
		gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook2), 0), label44);

  vbox32 = gtk_vbox_new (FALSE, 0);
  gtk_object_set_data (GTK_OBJECT (preference), "vbox32", vbox32);
  gtk_widget_show (vbox32);
  gtk_container_add (GTK_CONTAINER (notebook2), vbox32);

  checkbutton41 = gtk_check_button_new_with_label ("Auto Bunch");
  gtk_object_set_data (GTK_OBJECT (preference), "checkbutton41", checkbutton41);
  gtk_widget_show (checkbutton41);
  gtk_box_pack_start (GTK_BOX (vbox32), checkbutton41, FALSE, FALSE, 0);

  checkbutton42 = gtk_check_button_new_with_label ("Auto Save On Quit");
  gtk_object_set_data (GTK_OBJECT (preference), "checkbutton42", checkbutton42);
  gtk_widget_show (checkbutton42);
  gtk_box_pack_start (GTK_BOX (vbox32), checkbutton42, FALSE, FALSE, 0);

  checkbutton43 = gtk_check_button_new_with_label ("Auto Move Links");
  gtk_object_set_data (GTK_OBJECT (preference), "checkbutton43", checkbutton43);
  gtk_widget_show (checkbutton43);
  gtk_box_pack_start (GTK_BOX (vbox32), checkbutton43, FALSE, FALSE, 0);

  label45 = gtk_label_new ("Views");
  gtk_object_set_data (GTK_OBJECT (preference), "label45", label45);
  gtk_widget_show (label45);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook2),
		gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook2), 1), label45);

  vbox34 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox34);
  gtk_container_add (GTK_CONTAINER (notebook2), vbox34);

  checkbutton44 = gtk_check_button_new_with_label ("Show Grid");
  gtk_widget_show (checkbutton44);
  gtk_box_pack_start (GTK_BOX (vbox34), checkbutton44, FALSE, FALSE, 0);

  label46 = gtk_label_new ("Graph");
  gtk_widget_show (label46);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook2),
		gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook2), 2), label46);

  hbox6 = gtk_hbox_new (FALSE, 4);
  gtk_widget_show (hbox6);
  gtk_box_pack_start (GTK_BOX (vbox11), hbox6, FALSE, TRUE, 0);

  button7 = gtk_button_new_with_label ("Close");
  gtk_widget_show (button7);
  gtk_box_pack_end (GTK_BOX (hbox6), button7, FALSE, FALSE, 0);
  gtk_widget_set_usize (button7, 100, -2);

  button6 = gtk_button_new_with_label ("Apply");
  gtk_widget_show (button6);
  gtk_box_pack_end (GTK_BOX (hbox6), button6, FALSE, FALSE, 0);
  gtk_widget_set_usize (button6, 100, -2);

	gtk_signal_connect (GTK_OBJECT (button7), "clicked",
		GTK_SIGNAL_FUNC (on_Close_activate), this);
  gtk_signal_connect (GTK_OBJECT (button6), "clicked",
		GTK_SIGNAL_FUNC (on_Apply_activate), this);
}

void Pref::show()
{
	copy_to_widgets();
	gtk_widget_show(preference);
	gdk_window_raise(preference->window);
}

void Pref::hide()
{
	gtk_widget_hide(preference);
}

void Pref::load()
{
	FILE *f;
	struct passwd *p;
	char buf[256], field[256], value[256];

	p = getpwuid(getuid());
	sprintf(preferences_file, "%s/%s/preferences", p->pw_dir, HOME_DIR);

	f = fopen(preferences_file, "r");
	if (!f) { save(); return; }

	while(fgets(buf, 256, f) != NULL)
	{
		if (sscanf(buf, "%[^ ] %[^\n]", field, value)==2)
		{
			if (strcmp(field, "SERVER_DIRECTORY")==0)
				strcpy(server_directory, value);
			else
			if (strcmp(field, "AUTO_BUNCH")==0)
				auto_bunch = (strcmp(value, "yes")==0)?1:0;
			else
			if (strcmp(field, "SAVE_VIEW_ON_QUIT")==0)
				save_view_on_quit = (strcmp(value, "yes")==0)?1:0;
			else
			if (strcmp(field, "AUTO_MOVE_LINKS")==0)
				auto_move_links = (strcmp(value, "yes")==0)?1:0;
			else
			if (strcmp(field, "SHOW_GRID")==0)
				show_grid = (strcmp(value, "yes")==0)?1:0;
		}
	}

	fclose(f);

	// add to widgets
	copy_to_widgets();
}

void Pref::save()
{
	FILE *f;
	struct passwd *p;

	p = getpwuid(getuid());
	sprintf(preferences_file, "%s/%s/preferences", p->pw_dir, HOME_DIR);

	f = fopen(preferences_file, "w");
	if (!f) return;

	fprintf(f, "SERVER_DIRECTORY %s\n", server_directory);
	fprintf(f, "AUTO_BUNCH %s\n", (auto_bunch==1?"yes":"no"));
	fprintf(f, "SAVE_VIEW_ON_QUIT %s\n", (save_view_on_quit==1?"yes":"no"));
	fprintf(f, "AUTO_MOVE_LINKS %s\n", (auto_move_links==1?"yes":"no"));
	fprintf(f, "SHOW_GRID %s\n", (show_grid==1?"yes":"no"));

	fclose(f);
}

void Pref::copy_from_widgets()
{
	strcpy(server_directory, gtk_entry_get_text(GTK_ENTRY(entry1)));
	auto_bunch = (GTK_TOGGLE_BUTTON(checkbutton41)->active)?1:0;
	save_view_on_quit = (GTK_TOGGLE_BUTTON(checkbutton42)->active)?1:0;
	auto_move_links = (GTK_TOGGLE_BUTTON(checkbutton43)->active)?1:0;
	show_grid = (GTK_TOGGLE_BUTTON(checkbutton44)->active)?1:0;
}

void Pref::copy_to_widgets()
{
	gtk_entry_set_text(GTK_ENTRY(entry1), server_directory);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbutton41),
		(auto_bunch==1)?TRUE:FALSE);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbutton42),
		(save_view_on_quit==1)?TRUE:FALSE);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbutton43),
		(auto_move_links==1)?TRUE:FALSE);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbutton44),
		(show_grid==1)?TRUE:FALSE);
}

