/*****************************************************************************
 Copyright  Newcastle University
 Author      Paul Coates
 *****************************************************************************/
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <stdio.h>
#include "main.h"
#include "ipaddress.h"
#include "subnet.h"
#include "router.h"
#include "network.h"
#include "interface.h"
#include "misc.h"
#include "images/icon.xpm"

static void on_Close_activate(GtkWidget *w, gpointer data)
{
	((SubnetWindow*)data)->hide();
}

static gint delete_event(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	((SubnetWindow*)data)->hide();
	return (TRUE); 
}

void SubnetWindow::tags()
{
	Link *ln, *ln2, *ln3;
	Interface *in;
	Router *r;
	Network *n;
	Subnet *s;
	IPAddress *ip;

	main->files->clear_tags();
	if (main->files->objects==NULL) return;
	if (current_subnet != NULL)
	{
		for (ln=main->files->objects->routers.get_head(); ln!=NULL; ln=ln->next)
		{
			r = (Router*) ln->item;
			for (ln2=r->interfaces.get_head(); ln2!=NULL; ln2=ln2->next)
			{
				in = (Interface*) ln2->item;
				for (ln3=in->ip_addresses.get_head(); ln3!=NULL; ln3=ln3->next)
				{
					ip = (IPAddress*) ln3->item;
					if (in_subnet(*ip, current_subnet->ip, current_subnet->mask))
						r->selected = 1;
				}
			}
		}
		for (ln=main->files->objects->networks.get_head(); ln!=NULL; ln=ln->next)
		{
			n = (Network*) ln->item;
			for (ln2=n->subnets.get_head(); ln2!=NULL; ln2=ln2->next)
			{
				s = (Subnet*) ln2->item;
				if (s==current_subnet) n->selected=1;
			}
		}
	}
	main->monitor->draw();
}

static gint clist_compare_ip(GtkCList *cl, gconstpointer p1, gconstpointer p2)
{
	GtkCListRow *row;
	Subnet *sn, *s2;

	row = (GtkCListRow*) p1;
	sn = (Subnet*) row->data;
	row = (GtkCListRow*) p2;
	s2 = (Subnet*) row->data;

	return sn->ip.real_compare(s2->ip);
}

static void title_cb(GtkWidget *clist, gint column, gpointer data)
{
	SubnetWindow *snw = (SubnetWindow*) data;
	snw->current_sort = column;

	gtk_clist_set_sort_column (GTK_CLIST (clist), column);

	if (column==1)
	{
		gtk_clist_set_compare_func (GTK_CLIST (clist), clist_compare_ip);
	}else
		gtk_clist_set_compare_func (GTK_CLIST (clist), NULL);

	gtk_clist_sort (GTK_CLIST (clist));
}

static void subnet_list_cb(GtkWidget *clist, gint row, gint column, GdkEventButton *event, gpointer data)
{
	SubnetWindow *m = (SubnetWindow*) data;
	Subnet *sn = (Subnet*) gtk_clist_get_row_data(GTK_CLIST (clist), row);
	m->current_subnet = sn;
	m->tags();
}

static void subnet_list_cb2(GtkWidget *clist, gint row, gint column, GdkEventButton *event, gpointer data)
{
	SubnetWindow *m = (SubnetWindow*) data;
	m->current_subnet = NULL;
	m->tags();
}

SubnetWindow::SubnetWindow(Main *m)
{
	main = m;
	current_sort = 0;
	current_subnet = NULL;
	create();
}

SubnetWindow::~SubnetWindow()
{
}

void SubnetWindow::create()
{
  subnet = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (subnet), "Subnet Details");
  gtk_widget_realize (subnet);
	window_icon(subnet, icon_xpm);
	gdk_window_set_icon_name(subnet->window, "Subnet");
	gtk_signal_connect(GTK_OBJECT(subnet), "delete_event",
		GTK_SIGNAL_FUNC(delete_event), this);

  vbox9 = gtk_vbox_new (FALSE, 4);
  gtk_widget_show (vbox9);
  gtk_container_add (GTK_CONTAINER (subnet), vbox9);
  gtk_container_set_border_width (GTK_CONTAINER (vbox9), 4);

  hbox3 = gtk_hbox_new (FALSE, 4);
  gtk_widget_show (hbox3);
  gtk_box_pack_start (GTK_BOX (vbox9), hbox3, TRUE, TRUE, 0);

  scrolledwindow15 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow15);
  gtk_box_pack_start (GTK_BOX (hbox3), scrolledwindow15, TRUE, TRUE, 0);
  gtk_widget_set_usize (scrolledwindow15, 500, 180);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow15), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  clist14 = gtk_clist_new (2);
  gtk_widget_show (clist14);
  gtk_container_add (GTK_CONTAINER (scrolledwindow15), clist14);
  gtk_widget_set_usize (clist14, 307, -2);
  gtk_clist_set_column_width (GTK_CLIST (clist14), 0, 280);
  gtk_clist_set_column_width (GTK_CLIST (clist14), 1, 80);
  gtk_clist_column_titles_show (GTK_CLIST (clist14));
  gtk_signal_connect (GTK_OBJECT (clist14), "click_column",
		GTK_SIGNAL_FUNC (title_cb), this);
	gtk_signal_connect(GTK_OBJECT(clist14), "select_row",
		GTK_SIGNAL_FUNC(subnet_list_cb), this);
	gtk_signal_connect(GTK_OBJECT(clist14), "unselect_row",
		GTK_SIGNAL_FUNC(subnet_list_cb2), this);

  label22 = gtk_label_new ("Description");
  gtk_widget_show (label22);
  gtk_clist_set_column_widget (GTK_CLIST (clist14), 0, label22);
  gtk_misc_set_alignment (GTK_MISC (label22), 0, 0.5);

  label23 = gtk_label_new ("IP Address / Mask");
  gtk_widget_show (label23);
  gtk_clist_set_column_widget (GTK_CLIST (clist14), 1, label23);
  gtk_misc_set_alignment (GTK_MISC (label23), 0, 0.5);

  hbox4 = gtk_hbox_new (FALSE, 4);
  gtk_widget_show (hbox4);
  gtk_box_pack_start (GTK_BOX (vbox9), hbox4, FALSE, TRUE, 0);

  button5 = gtk_button_new_with_label ("Close");
  gtk_widget_show (button5);
  gtk_box_pack_end (GTK_BOX (hbox4), button5, FALSE, FALSE, 0);
  gtk_widget_set_usize (button5, 100, -2);

  gtk_signal_connect (GTK_OBJECT (button5), "clicked", GTK_SIGNAL_FUNC (on_Close_activate), this);
}

void SubnetWindow::show()
{
	init();
	gtk_widget_show(subnet);
	gdk_window_raise(subnet->window);
}

void SubnetWindow::hide()
{
	gtk_widget_hide(subnet);
}

void SubnetWindow::init()
{
	Link *ln;
	gchar *data[2];
	char buf[256], buf2[256];
	Subnet *s;
	int row;

	if (main->files->objects==NULL) return;

	gtk_clist_clear(GTK_CLIST(clist14));

	row = 0;
	for (ln=main->files->objects->subnets.get_head(); ln!=NULL; ln=ln->next)
	{
		s = (Subnet*) ln->item;
		data[0] = strdup(s->name);
		s->ip.sprint(buf); sprintf(buf2, "%s/%d", buf, s->mask);
		data[1] = strdup(buf2);
		gtk_clist_append(GTK_CLIST(clist14), data);
		delete data[0]; delete data[1];
		gtk_clist_set_row_data(GTK_CLIST(clist14), row, s);
		row++;
	}

	sprintf(buf, "Description (%d)", row);
	gtk_clist_set_column_title(GTK_CLIST(clist14), 0, buf);

	title_cb(clist14, current_sort, this);
}

