/*****************************************************************************
 Copyright  Newcastle University
 Author      Paul Coates
 *****************************************************************************/
#include <stdio.h>
#include "list.h"

List::List()
{
	count = 0;
	first = NULL;
	last = NULL;
}

List::~List()
{
	clear();
}

void List::add(void *i)
{
	Link *lnk = new Link();
	lnk->item = i;
	lnk->next = NULL;
	lnk->prev = NULL;

	if (first == NULL)
	{
		first = last = lnk;
	}else
	{
		first->prev = lnk;
		lnk->next = first;
		first = lnk;
	}
	count++;
}

void List::tag(void *i)
{
	Link *lnk = new Link();
	lnk->item = i;
	lnk->next = NULL;
	lnk->prev = NULL;

	if (last == NULL)
	{
		first = last = lnk;
	}else
	{
		last->next = lnk;
		lnk->prev = last;
		last = lnk;
	}
	count++;
}

void List::del(void *i)
{
	Link *lk, *lnk = first;

	while (lnk!=NULL)
	{
		lk = lnk->next;
		if (i == lnk->item)
		{
			if (first == lnk) first = lnk->next;
			if (last == lnk) last = lnk->prev;
			if (lnk->prev != NULL) lnk->prev->next = lnk->next;
			if (lnk->next != NULL) lnk->next->prev = lnk->prev;
			delete lnk;
			count--;
		}
		lnk = lk;
	}
}

void List::clear()
{
	Link *lnk = first;

	while (lnk!=NULL)
	{
		lnk = first->next;
		delete first;
		first = lnk;
	}
	last = NULL;
	count = 0;
}

void List::raise(void *i)
{
	del(i);
	add(i);
}

void List::lower(void *i)
{
	del(i);
	tag(i);
}

void *List::get_item(int i)
{
	Link *current = first;
	while(i>0)
	{
		if (current) current = current->next;
		i--;
	}
	if (current) return current->item;
	return NULL;
}

Link *List::get_head()
{
	return first;
}

Link *List::get_tail()
{
	return last;
}

int List::size()
{
	return count;
}

void swap(Link *a, Link *b)
{
	void *c = a->item;
	a->item = b->item;
	b->item = c;	
}

void List::sort(Compare compare, int type)
{
	switch (type)
	{
		case BUBBLE_SORT: bubblesort(compare, first, last); break;
		default: break;
	}
}

void List::bubblesort(Compare compare, Link *low, Link *high)
{
	Link *ln, *ln2;

	if (high==NULL) return;
	if (low==NULL) return;
	if (low==high) return;

	for (ln=low; ln!=NULL; ln=ln->next)
	{
		for (ln2=ln->next; ln2!=NULL; ln2=ln2->next)
		{
				if (compare(ln, ln2)>0) swap(ln, ln2);
		}
	}
}

