/*****************************************************************************
 Copyright  Newcastle University
 Author      Paul Coates
 *****************************************************************************/
#include <stdlib.h>
#include <string.h>
#include "subnet.h"
#include "misc.h"

Subnet::Subnet()
{
	clear();
}

Subnet::~Subnet()
{
}

void Subnet::clear()
{
	int i;

	strcpy(name, "");
	ip.clear();
	gateway.clear();
	mask = 0;
	find = 0;
}

void Subnet::save(FILE *f)
{
	int i;

	for (i=0; i<64; i++)  fputc(name[i], f);
	gateway.save(f);
	ip.save(f);
	write_int(f, mask);
}

void Subnet::load(FILE *f)
{
	int i;

	for (i=0; i<64; i++) name[i] = fgetc(f); name[64] = '\0';
	gateway.load(f);
	ip.load(f);
	mask = read_int(f);
}

void Subnet::print()
{
	printf("Subnet name %s, ", name);
	gateway.print();
	printf(", ");
	ip.print();
	printf(" / %d\n", mask);
}

void Subnet::match(char *string)
{
	char nm[256], tmpnm[256];

	find = 0;
	if (strlen(string)==0) return;

	ip.sprint(tmpnm);
	sprintf(nm, "%s / %d", tmpnm, mask);
	if ((strlen(name)>0) && (lower_contains(name, string))) find = 1; else
	if (lower_contains(nm, string)) find = 2;
}

/////////////////////////////////////////////////////
// Save / Load XML
//

void Subnet::savexml(xmlNodePtr xmlnode)
{
	xmlNodePtr newxml;
	char buf[256];

	newxml = xmlNewChild(xmlnode, NULL, (xmlChar*)"Subnet", NULL);
	xmlSetProp(newxml, (xmlChar*)"name", (xmlChar*)name);
	gateway.sprint(buf);
	xmlSetProp(newxml, (xmlChar*)"gateway", (xmlChar*)buf);
	ip.sprint(buf);
	xmlSetProp(newxml, (xmlChar*)"ip", (xmlChar*)buf);
	sprintf(buf, "%d", mask);
	xmlSetProp(newxml, (xmlChar*)"mask", (xmlChar*)buf);
}

void Subnet::loadxml(xmlNodePtr xmlnode)
{
	xmlChar *buf;

	if (!xmlnode->name || strcmp((char*)xmlnode->name, "Subnet")!=0)
		return;

	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"name");
	strcpy(name, (char*)buf);
	free(buf);

	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"gateway");
	gateway.set((char*)buf);
	free(buf);

	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"ip");
	ip.set((char*)buf);
	free(buf);

	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"mask");
	sscanf((char*)buf, "%d", &mask);
	free(buf);
}

