/*****************************************************************************
 Copyright  Newcastle University
 Author      Paul Coates
 *****************************************************************************/
#include <unistd.h>
#include <stdio.h>
#include <pwd.h>
#include "main.h"
#include "misc.h"
#include "topology.h"
#include "ping.h"

int main(int argc, char *argv[])
{
	Main m;

	if (argc>1)
	{
		switch (argv[1][0])
		{
			case 't':
				Topology *tp;
				tp = new Topology();
				tp->main();
				delete tp;
				break;

			case 'p':
				Ping *pg;
				pg = new Ping();
				pg->main();
				delete pg;
				break;

			default: break;
		}
	}else
	{
		m.setup();
		m.run();
		m.quit();
	}

	return 0;
}

void Main::setup()
{
	Process *p;
	int i;

	for (i=1; i<=NUM_PROCESSES; i++)
	{
		p = new Process();
		p->create();
		processes.tag(p);
	}
}

void Main::quit()
{
	Link *ln;

	for (ln=processes.get_head(); ln!=NULL; ln=ln->next)
		((Process*)ln->item)->send("quit");
}

// check the task list for the task that is being run by the returning process
void Main::check_incoming_task(List *lst, Process *p)
{
	Link *ln2;
	Task *t;

	for (ln2=lst->get_head(); ln2!=NULL; ln2=ln2->next)
	{
		t = (Task*) ln2->item;
		if (t->pid == p->pid)
		{
			t->pid = 0;
			t->status = 1;
		}
	}
}

// the main server loop
void Main::run()
{
	char buf[256];
	List stat_tsk;
	List top_tsk;
	List gen_graph_tsk;
	List gen_network_tsk;
	List gen_ping_tsk;
	Process *p;
	Link *ln, *ln2;
	Task *t;
	int flag;

	stat_tm = top_tm = ping_tm = time(NULL);
	stat_mode = top_mode = ping_mode = 0;

	t = new Task();
	strcpy(t->cmnd, "gen graph");
	gen_graph_tsk.tag(t);

	t = new Task();
	strcpy(t->cmnd, "gen network");
	gen_network_tsk.tag(t);

	t = new Task();
	strcpy(t->cmnd, "nomad_alive");
	gen_ping_tsk.tag(t);

	while(1)
	{
		// check for processes ending and set appropriate flags
		for (ln=processes.get_head(); ln!=NULL; ln=ln->next)
		{
			p = (Process*) ln->item;
			if (p->get(buf))
			{
				check_incoming_task(&stat_tasks, p);
				check_incoming_task(&top_tasks, p);
				check_incoming_task(&gen_graph_tsk, p);
				check_incoming_task(&gen_network_tsk, p);
				check_incoming_task(&gen_ping_tsk, p);
//				printf("RETURNED %s\n", buf);
				p->tid = 0;
			}
		}

		if (ping_mode == 0)
		{
			if (ping_tm<time(NULL))
			{
				ln2=gen_ping_tsk.get_head();
				t = (Task*) ln2->item;
				t->status = 0;
				ping_mode = 1;
			}
		}else
		if (ping_mode == 1)
		{
			ln2=gen_ping_tsk.get_head();
			t = (Task*) ln2->item;
			if (t->status == 0)
				process_tasks(&gen_ping_tsk);
			else
			{
				ping_tm = time(NULL)+180;
				ping_mode = 0;
			}
		}

		if (stat_mode == 0)		// wait -> read devices & create task list
		{
			if (stat_tm<time(NULL))
			{
				tasks_from_devices(&stat_tasks, "nomad_statistics");
				stat_mode = 1;
			}
		}else
		if (stat_mode == 1)		// grab stats
		{
			flag = 0;
			for (ln2=stat_tasks.get_head(); ln2!=NULL; ln2=ln2->next)
			{
				t = (Task*) ln2->item;
				if (t->status == 0) flag = 1;
			}
			if (flag == 0)	// all processes returned
			{
				ln2=gen_graph_tsk.get_head();
				t = (Task*) ln2->item;
				t->status = 0;
				stat_mode = 2;
			}else
				process_tasks(&stat_tasks);
		}else
		if (stat_mode == 2)
		{
			ln2=gen_graph_tsk.get_head();
			t = (Task*) ln2->item;
			if (t->status == 0)
				process_tasks(&gen_graph_tsk);
			else
			{
				stat_tm = time(NULL)+300;
				stat_mode = 0;
			}
		}

		if (top_mode == 0)		// wait -> read devices & create task list
		{
			if (top_tm<time(NULL))
			{
				tasks_from_devices(&top_tasks, "nomad_topology");
				top_mode = 1;
			}
		}else
		if (top_mode == 1)		// grab stats
		{
			flag = 0;
			for (ln2=top_tasks.get_head(); ln2!=NULL; ln2=ln2->next)
			{
				t = (Task*) ln2->item;
				if (t->status == 0) flag = 1;
			}
			if (flag == 0)
			{
				ln2=gen_network_tsk.get_head();
				t = (Task*) ln2->item;
				t->status = 0;
				top_mode = 2;
			}else
				process_tasks(&top_tasks);
		}else
		if (top_mode == 2)
		{
			ln2=gen_network_tsk.get_head();
			t = (Task*) ln2->item;
			if (t->status == 0)
				process_tasks(&gen_network_tsk);
			else
			{
				top_tm = time(NULL)+1200;
				top_mode = 0;
			}
		}
		// make a list of unchecked ip addresses from the network file
		// for each unchecked ip
		//   ping it then try snmpget and tag successful matches
		// update devices file with new matches
		// fping all devices in the devices file
status();
		usleep(2000000);
	}

	ln2=gen_network_tsk.get_head();
	t = (Task*) ln2->item;
	delete t;

	ln2=gen_graph_tsk.get_head();
	t = (Task*) ln2->item;
	delete t;
}

// create a task list from the devices file
void Main::tasks_from_devices(List *lst, char *str)
{
	struct passwd *p;
	char mapdir[256];
	char buf[256];
	char buf2[256];
	char buf3[256];
	char buf4[256];
	char line[256];
	FILE *f;
	Task *t;
	Link *ln;

	// delete any existing tasks
	for (ln=lst->get_head(); ln!=NULL; ln=ln->next)
	{
		t = (Task*) ln->item;
		delete t;
	}
	lst->clear();

	// create a new list
	p = getpwuid(getuid());
	sprintf(mapdir, "%s/%s", p->pw_dir, HOME_DIR);
	sprintf(buf, "%s/devices", mapdir);
	f = fopen(buf, "r");
	if (f)
	{
		while(fgets(line, 256, f) != NULL)
		{
			if (sscanf(line, "%[^,],%[^,],%[^\n]\n", buf2, buf3, buf4)==3)
			{
				t = new Task();
				sprintf(t->cmnd, "%s %s %s %s", str, buf2, buf3, buf4);
				lst->tag(t);
			}
		}
		fclose(f);
	}
}

// take in a task list and assign what you can to any free processes
void Main::process_tasks(List *lst)
{
	Link *ln, *ln2;
	Process *p;
	Task *t;

	for (ln=processes.get_head(); ln!=NULL; ln=ln->next)
	{
		p = (Process*)ln->item;
		if (p->tid==0)
		{
			ln2=lst->get_head();
			while(ln2!=NULL)
			{
				t = (Task*) ln2->item;
				if ((t->status==0) && (t->pid==0))
				{
					t->pid = p->pid;
					p->tid = 1;
					strcpy(p->cmnd, t->cmnd);
					p->send(t->cmnd);
					ln2=NULL;
				}else
				{
					ln2=ln2->next;
				}
			}
		}
	}
}

void Main::status()
{
	Link *ln;
	Process *p;
	int i;

	i=1;
	for (ln=processes.get_head(); ln!=NULL; ln=ln->next)
	{
		p = (Process*)ln->item;
		printf("%2d %6d ", i, p->pid);
		if (p->tid!=0)
		{
			printf("%s", p->cmnd);
		}
		printf("\n");
		i++;
	}
}

