/*****************************************************************************
 Copyright  Newcastle University
 Author      Paul Coates
 *****************************************************************************/
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <pwd.h>
#include "topology.h"
#include "machine.h"
#include "interface.h"
#include "network.h"
#include "router.h"
#include "subnet.h"
#include "device.h"
#include "port.h"
#include "iface.h"
#include "route.h"
#include "mac.h"
#include "address.h"
#include "misc.h"
#include "join.h"
#include "ipaddr.h"
#include "ipaddress.h"

Topology::Topology()
{
	num_devices = 0;
	num_ports = 0;
	dv = NULL;
	pt = NULL;
	mm = 1;
}

Topology::~Topology()
{
	if (dv!=NULL) delete [] dv;
	if (pt!=NULL) delete [] pt;
}

void Topology::main()
{
	FILE *f;
	struct passwd *p;
	char buf[256], host[256], ip[256], comm[256];

	p = getpwuid(getuid());
	sprintf(mapdir, "%s/%s", p->pw_dir, HOME_DIR);

	// load the new topology files into memory
	sprintf(buf, "%s/devices", mapdir);
	f = fopen(buf, "r");
	if (f)
	{
		while(fscanf(f, "%[^\n]\n", buf)==1) num_devices++;
		fseek(f, 0, SEEK_SET);
		dv = new Device[num_devices];
		pos = 0;
		while(fscanf(f, "%[^\n]\n", buf)==1)
		{
			sscanf(buf, "%[^,],%[^,],%[^\n]", host, ip, comm);
			sprintf(buf, "%s/topology/%s", mapdir, host);
			strcpy(dv[pos].name, host);
			strcpy(dv[pos].community, comm);
			dv[pos].pingname.set(ip);
			load(buf);
			pos++;
		}
		fclose(f);
	}
	build_network();
	build_machines();
	merge();
	save();
}

/*****************************************************************************
 *
 *  This section has all the code needed to load the topology files
 *
 *****************************************************************************/

void Topology::load(char *fname)
{
	FILE *f;
	char buf[256], var[256], val[256];

	int a, b, c, d, e, g, i, s;

	f = fopen(fname, "r");
	if (!f) return;

	// work out how big the tables are
	a = b = c = d = e = g = 0;
	while(fscanf(f, "%[^\n]\n", buf)==1)
	{
		if (sscanf(buf, "%[^ ] %[^\n]", var, val)==2)
		{
			if (starts_with(".iso.3.6.1.2.1.2.2.1.5", var)) a++;
			else	// routing table
			if (starts_with(".iso.3.6.1.2.1.4.20.1.2", var)) g++;
			else
			if (starts_with(".iso.3.6.1.2.1.4.21.1.2", var)) b++;
			else	// address table
			if (starts_with(".iso.3.6.1.2.1.3.1.1.1", var)) c++;
			else	// bridge map ports
			if (starts_with(".iso.3.6.1.2.1.17.1.4.1.2", var)) d++;
			else	// bridge tables
			if (starts_with(".iso.3.6.1.2.1.17.4.3.1.2", var)) e++;
		}
	}

	dv[pos].num_interfaces = a;
	dv[pos].num_addresses = c;
	dv[pos].num_macs = e;
	dv[pos].num_routes = b;
	dv[pos].num_ips = g;
	dv[pos].type = 'u';
	dv[pos].interfaces = new Iface[a];
	dv[pos].addresses = new Address[c];
	dv[pos].macs = new Mac[e];
	dv[pos].routes = new Route[b];
	dv[pos].ips = new IPAddr[g];

// printf("%d,%d,%d,%d,%d,%d %s\n", a, b, c, d, e, g, fname);

	if (pt != NULL) delete [] pt;
	pt = new Port[d];
	num_ports = d;
	for (i=0; i<num_ports; i++) pt[i].index = -1;

	// the second pass
	fseek(f, 0, SEEK_SET);

	fscanf(f, "%[^\n]\n", buf);
	dv[pos].grab_time = make_time_from_string(buf);

	while(fscanf(f, "%[^\n]\n", buf)==1)
	{
		if (sscanf(buf, "%[^ ] %[^\n]", var, val)==2)
		{
			if (strcmp(".iso.3.6.1.2.1.1.7.0", var)==0)
			{
				if (sscanf(val, "%d", &s)==1)
				{
					if ((s&6)==6) dv[pos].type = 'r'; else
					if ((s&6)==2) dv[pos].type = 's';
				}
			}else
			if (starts_with(".iso.3.6.1.2.1.2.2.1", var))
			{
				add_to_interface(var, val);
			}else
			if (starts_with(".iso.3.6.1.2.1.31.1.1.1", var))
			{
				add_to_interface_extended(var, val);
			}else
			if (starts_with(".iso.3.6.1.2.1.4.21.1", var))
			{
				if (dv[pos].type == 'r') add_to_route(var, val);
			}else
			if (starts_with(".iso.3.6.1.2.1.4.20.1", var))
			{
				add_to_ips(var, val);
			}else
			if (starts_with(".iso.3.6.1.2.1.3.1.1", var))
			{
				add_to_address(var, val);
			}else
			if (starts_with(".iso.3.6.1.2.1.17.1.4.1", var))
			{
				add_bridge_map(var, val);
			}else
			if (starts_with(".iso.3.6.1.2.1.17.4.3.1", var))
			{
				add_to_bridge(var, val);
			}
		}
	}

	fclose(f);

	// map the ports to the device interfaces
	port_to_if();

/*
	printf("%s\n", dv[pos].name);
	for (i=0; i<dv[pos].num_routes; i++)
	{
		printf("ROUTE INDEX %d, MASK %d, TYPE %d, DEST ", dv[pos].routes[i].index,
			dv[pos].routes[i].mask, dv[pos].routes[i].type);
		dv[pos].routes[i].dest.print();
		printf("\n");
	}
	for (i=0; i<dv[pos].num_ips; i++)
	{
		printf("IP INDEX %d, IP ", dv[pos].ips[i].index);
		dv[pos].ips[i].ip.print();
		printf("\n");
	}
	for (i=0; i<dv[pos].num_addresses; i++)
	{
		printf("ADDRESS INDEX %d, IP ", dv[pos].addresses[i].index);
		dv[pos].addresses[i].ip.print();
		printf(", MAC ");
		print_mac(dv[pos].addresses[i].mac);
		printf("\n");
	}
*/
}

void Topology::add_to_interface(char *var, char *val)
{
	char vartype[256], junk[256];
	int i, j, ifnum, iffound = -1, ifgap = -1;
	unsigned long s;

	// an interface variable is split into 3
	if (sscanf(var, ".iso.3.6.1.2.1.2.2.1.%[^.].%d", vartype, &ifnum)!=2)
		return;
	// look for an existing interface entry with the same ifnum
	for (i=0; i<dv[pos].num_interfaces; i++)
	{
		if (dv[pos].interfaces[i].index == ifnum) iffound = i;
		if ((dv[pos].interfaces[i].index == -1) && (ifgap == -1)) ifgap = i;
	}

	if (iffound == -1) // failed to find interface
	{
		if (ifgap != -1) // found a gap
		{
			iffound = ifgap;
			dv[pos].interfaces[iffound].index = ifnum;
		}else
			return;
	}
	if (strcmp("5", vartype)==0)
	{
		if (sscanf(val, "%lu", &s)==1)
			dv[pos].interfaces[iffound].speed = s;
	}else
	if (strcmp("2", vartype)==0)
	{
		if (sscanf(val, "\"%[^\"]\"", junk)==1)
			strcpy(dv[pos].interfaces[iffound].descr, junk);
	}else
	if (strcmp("3", vartype)==0)
	{
		if (sscanf(val, "%d", &j)==1)
			dv[pos].interfaces[iffound].type = j;
	}else
	if (strcmp("8", vartype)==0)
	{
		if (sscanf(val, "%d", &j)==1)
			dv[pos].interfaces[iffound].status = j;
	}else
	if (strcmp("6", vartype)==0)
	{
		convert_mac(val, dv[pos].interfaces[iffound].mac);
	}
}

void Topology::add_to_interface_extended(char *var, char *val)
{
	char vartype[256], junk[256];
	int i, ifnum, iffound = -1, ifgap = -1;

	// an interface variable is split into 3
	if (sscanf(var, ".iso.3.6.1.2.1.31.1.1.1.%[^.].%d", vartype, &ifnum)!=2)
		return;
	// look for an existing interface entry with the same ifnum
	for (i=0; i<dv[pos].num_interfaces; i++)
	{
		if (dv[pos].interfaces[i].index == ifnum) iffound = i;
		if ((dv[pos].interfaces[i].index == -1) && (ifgap == -1)) ifgap = i;
	}

	if (iffound == -1) // failed to find interface
	{
		if (ifgap != -1) // found a gap
		{
			iffound = ifgap;
			dv[pos].interfaces[iffound].index = ifnum;
		}else
			return;
	}
	if (strcmp("1", vartype)==0)
	{
		if (sscanf(val, "\"%[^\"]\"", junk)==1)
			if (strlen(junk)>0)
				strcpy(dv[pos].interfaces[iffound].port, junk);
	}
}

void Topology::add_to_route(char *var, char *val)
{
	char vartype[256], idxval[256];
	IPAddress tmpip;
	int i, j, iffound = -1, ifgap = -1;

	// a route variable is split into 3
	if (sscanf(var, ".iso.3.6.1.2.1.4.21.1.%[^.].%[^\n]", vartype, idxval)!=2)
		return;
	tmpip.set(idxval);
	// look for an existing interface entry with the same ifnum
	for (i=0; i<dv[pos].num_routes; i++)
	{
		if (dv[pos].routes[i].dest.compare(tmpip)) iffound = i;
		if ((dv[pos].routes[i].dest.zero()) && (ifgap == -1)) ifgap = i;
	}

	if (iffound == -1) // failed to find interface
	{
		if (ifgap != -1) // found a gap
		{
			iffound = ifgap;
			dv[pos].routes[iffound].dest.set(tmpip);
		}else
			return;
	}
	if (strcmp("2", vartype)==0)
	{
		if (sscanf(val, "%d", &j)==1)
			dv[pos].routes[iffound].index = j;
	}else
	if (strcmp("7", vartype)==0)
	{
		dv[pos].routes[iffound].next.set(val);
	}else
	if (strcmp("11", vartype)==0)
	{
		dv[pos].routes[iffound].mask = mask_num(val);
	}else
	if (strcmp("8", vartype)==0)
	{
		if (sscanf(val, "%d", &j)==1)
			dv[pos].routes[iffound].type = j;
	}
}

void Topology::add_to_ips(char *var, char *val)
{
	char vartype[256], idxval[256];
	int i, j, iffound = -1, ifgap = -1;
	IPAddress tmpip;

	// an ips variable is split into 3
	if (sscanf(var, ".iso.3.6.1.2.1.4.20.1.%[^.].%[^\n]", vartype, idxval)!=2)
		return;
	tmpip.set(idxval);
	// look for an existing interface entry with the same ifnum
	for (i=0; i<dv[pos].num_ips; i++)
	{
		if (dv[pos].ips[i].ip.compare(tmpip)) iffound = i;
		if ((dv[pos].ips[i].ip.zero()) && (ifgap == -1)) ifgap = i;
	}

	if (iffound == -1) // failed to find interface
	{
		if (ifgap != -1) // found a gap
		{
			iffound = ifgap;
			dv[pos].ips[iffound].ip.set(tmpip);
		}else
			return;
	}
	if (strcmp("2", vartype)==0)
	{
		if (sscanf(val, "%d", &j)==1)
			dv[pos].ips[iffound].index = j;
	}
}

void Topology::add_to_address(char *var, char *val)
{
	char vartype[256], idxval[256], j1[256], j2[256];
	IPAddress tmpip;
	int i, j, iffound = -1, ifgap = -1;

	// an address variable is split into 5
	if (sscanf(var, ".iso.3.6.1.2.1.3.1.1.%[^.].%[^.].%[^.].%[^\n]", vartype, j1, j2, idxval)!=4)
		return;
	tmpip.set(idxval);
	// look for an existing interface entry with the same ifnum
	for (i=0; i<dv[pos].num_addresses; i++)
	{
		if (dv[pos].addresses[i].ip.compare(tmpip)) iffound = i;
		if ((dv[pos].addresses[i].ip.zero()) && (ifgap == -1)) ifgap = i;
	}

	if (iffound == -1) // failed to find interface
	{
		if (ifgap != -1) // found a gap
		{
			iffound = ifgap;
			dv[pos].addresses[iffound].ip.set(tmpip);
		}else
			return;
	}

	if (strcmp("1", vartype)==0)
	{
		if (sscanf(val, "%d", &j)==1)
			dv[pos].addresses[iffound].index = j;
	}else
	if (strcmp("2", vartype)==0)
	{
		convert_mac(val, dv[pos].addresses[iffound].mac);
	}
}

void Topology::add_bridge_map(char *var, char *val)
{
	int i, j, ifnum, iffound = -1, ifgap = -1;

	// a bridge map variable is split into 2
	if (sscanf(var, ".iso.3.6.1.2.1.17.1.4.1.2.%d", &ifnum)!=1) return;
	sscanf(val, "%d", &j);
	for (i=0; i<num_ports; i++)
	{
		if (pt[i].index == ifnum) iffound = i;
		if ((pt[i].index == -1) && (ifgap == -1)) ifgap = i;
	}
	if (iffound == -1) // failed to find interface
	{
		if (ifgap != -1) // found a gap
		{
			iffound = ifgap;
			pt[iffound].index = ifnum;
		}else
			return;
	}
	pt[iffound].port = j;
}

void Topology::add_to_bridge(char *var, char *val)
{
	unsigned char vartype[6];
	char newmac[256];
	int i, j, iffound = -1, ifgap = -1;

	// a bridge variable is split into 2
	if (sscanf(var, ".iso.3.6.1.2.1.17.4.3.1.2.%[^\n]", newmac)!=1) return;
	convert_mac_2(newmac, vartype);
	for (i=0; i<dv[pos].num_macs; i++)
	{
		if (compare_mac(dv[pos].macs[i].mac, vartype)==1) iffound = i;
		if (zero_mac(dv[pos].macs[i].mac) && (ifgap == -1)) ifgap = i;
	}
	if (iffound == -1) // failed to find interface
	{
		if (ifgap != -1) // found a gap
		{
			iffound = ifgap;
			for (i=0; i<6; i++) dv[pos].macs[iffound].mac[i] = vartype[i];
		}else
			return;
	}
	sscanf(val, "%d", &j);
	dv[pos].macs[iffound].port = j;
}

void Topology::port_to_if()
{
	int i, j, tmp;

	for (i=0; i<dv[pos].num_macs; i++)
	{
		tmp = dv[pos].macs[i].port;
		for (j=0; j<num_ports; j++)
		{
			if (tmp == pt[j].index)
				dv[pos].macs[i].port = pt[j].port;
		}
	}
}


/*****************************************************************************
 *
 *  This section builds a topology diagram from the new data
 *
 *****************************************************************************/

Device *Topology::get_device(Router *r)
{
	int i;

	if (r==NULL) return NULL;

	for (i=0; i<num_devices; i++)
	{
		if (strcmp(r->name, dv[i].name)==0)
			return &dv[i];
	}
	return NULL;
}

Iface *Topology::get_interface(Interface *in)
{
	int i;
	Device *d;

	if (in==NULL) return NULL;

	d = get_device(in->router);
	if (d==NULL) return NULL;
	for (i=0; i<d->num_interfaces; i++)
	{
		if (in->index==d->interfaces[i].index)
			return &(d->interfaces[i]);
	}
	return NULL;
}

void Topology::add_subnet(Subnet *s)
{
	Link *ln;
	Subnet *s2, *s3;
	int got;

	if (s->mask > 30) return;

	got = 0;
	for (ln=newdata.subnets.get_head(); ln!=NULL; ln=ln->next)
	{
		s2 = (Subnet*) ln->item;
		if ((s->ip.compare(s2->ip)) && (s2->mask == s->mask)) got++;
	}

	if (!got)
	{
		s3 = new Subnet();
		s3->ip.set(s->ip);
		s3->gateway.set(s->gateway);
		s3->mask = s->mask;
		newdata.subnets.tag(s3);
	}
}

void Topology::build_network()
{
	int i, j, k, l, num, count;
	Interface *in;
	Network *n, *n2;
	Device *d;
	Router *r, *r2;
	Iface *in2, *in3;
	Interface *in4, *in5;
	Link *ln, *ln2, *ln3, *ln4, *ln5;
	IPAddress *tip, tip1, tip2;
	List tricky;

	// start by building the basic structure, all devices are created as machines,
	// interface objects and network objects are created and attached

	for (i=0; i<num_devices; i++)
	{
		r = new Router();
		r->pingname.set(dv[i].pingname);
		strcpy(r->name, dv[i].name);
		strcpy(r->community, dv[i].community);
		r->last_seen = dv[i].grab_time;
		// location is user info so don't overwrite
		r->type = dv[i].type;
		for (j=0; j<dv[i].num_interfaces; j++)
		{
			in = new Interface();
			in->index = dv[i].interfaces[j].index;
			for (l=0; l<6; l++) in->mac[l] = dv[i].interfaces[j].mac[l];
			in->speed = dv[i].interfaces[j].speed;
			in->type = dv[i].interfaces[j].type;
			in->status = dv[i].interfaces[j].status;
			strcpy(in->descr, dv[i].interfaces[j].descr);
			if (strcmp(dv[i].interfaces[j].port, "")!=0)
				strcpy(in->port, dv[i].interfaces[j].port);
			else
				sprintf(in->port, "%s (%d)", in->descr, in->index);
			// add the list of ip addresses
			for (k=0; k<dv[i].num_ips; k++)
			{
				if (dv[i].ips[k].index == in->index)
				{
					tip = new IPAddress();
					tip->set(dv[i].ips[k].ip);
					in->ip_addresses.tag(tip);
				}
			}

			in->router = r;
			n = new Network();
			n->interfaces.tag(in);
			in->network = n;
			newdata.networks.tag(n);
			r->interfaces.tag(in);
		}
		newdata.routers.tag(r);
	}

	// now go through the routing tables and make a list of subnets
	// in each network

	for (ln=newdata.routers.get_head(); ln!=NULL; ln=ln->next)
	{
		r = (Router*) ln->item;
		d = get_device(r);		// get a pointer to the equivalent device
		if (d!=NULL)
		{
			for (ln2=r->interfaces.get_head(); ln2!=NULL; ln2=ln2->next)
			{
				in = (Interface*) ln2->item;
				in2 = get_interface(in);
				num = 0;
				for (k=0; k<d->num_routes; k++)
				{
					if ((d->routes[k].index == in->index) &&
							(d->routes[k].type == 3) &&
							(!(d->routes[k].dest.in_range("127.0.0.0", "127.0.0.255"))) &&
							(d->routes[k].dest.zero()==0))
					{
						num++;
					}
				}
				in2->subnets = new Subnet[num];
				in2->num_subnets = num;
				num = 0;
				for (k=0; k<d->num_routes; k++)
				{
					if ((d->routes[k].index == in->index) &&
							(d->routes[k].type == 3) &&
							(!(d->routes[k].dest.in_range("127.0.0.0", "127.0.0.255"))) &&
							(d->routes[k].dest.zero()==0))
					{
						in2->subnets[num].ip.set(d->routes[k].dest);
						in2->subnets[num].mask = d->routes[k].mask;
						for (i=0; i<d->num_ips; i++)
						{
							tip2.set(d->ips[i].ip);
							if (in_subnet(tip2, d->routes[k].dest, d->routes[k].mask))
							{
								in2->subnets[num].gateway.set(d->ips[i].ip);
							}
						}
						add_subnet(&in2->subnets[num]);
						num++;
					}
				}
			}
		}
	}

	// compare network objects, if two share the same subnets they are
	// the same network and merge them. The following code is rather
	// messy but it does make the data structures easier to handle

	// NOTE: we are deleting networks while going through the list
	// so we have to check for NULL pointers

	for (ln=newdata.networks.get_head(); ln!=NULL; ln=ln->next)
	{
		for (ln2=newdata.networks.get_head(); ln2!=NULL; ln2=ln2->next)
		{
			n = (Network*) ln->item;
			n2 = (Network*) ln2->item;
			if ((n!=NULL) && (n2!=NULL) && (n != n2))
			{
				// a network has at least one interface attached so pick the first one
				ln5 = n->interfaces.get_head();
				in5 = (Interface*) ln5->item;
				in2 = get_interface(in5);

				ln5 = n2->interfaces.get_head();
				in5 = (Interface*) ln5->item;
				in3 = get_interface(in5);
				// in2 and in3 are Iface objects which include subnet info
				if ((in2->num_subnets > 0) && (in3->num_subnets > 0))
				{
					count = 0;
					for (k=0; k<in2->num_subnets; k++)
					{
						if (in2->subnets[k].ip.compare(in3->subnets[0].ip) &&
								(in2->subnets[k].mask == in3->subnets[0].mask))
							count = 1;
					}
					if (count == 1)
					{
						// both same so merge the networks
						for (ln5=n->interfaces.get_head(); ln5!=NULL; ln5=ln5->next)
						{
							in = (Interface*) ln5->item;
							n2->interfaces.tag(in);
							in->network = n2;
						}
						ln4 = ln->next;
						newdata.networks.del(n);
						delete n;
						ln = ln4;
					}
				}
			}
		}
	}

	// at this point we have a set of routers connected together via networks
	// now we need to check for any switches


	for (ln=newdata.routers.get_head(); ln!=NULL; ln=ln->next)
	{
		r = (Router*) ln->item;
		if (r->type == 'r')
		{
			for (ln2=r->interfaces.get_head(); ln2!=NULL; ln2=ln2->next)
			{
				in = (Interface*) ln2->item;
				// got an interface on a router so we know the MAC address
				// of the interface, look at the bridging tables on each
				// switch looking for the same MAC address
//				printf("Router interface %s / %s has MAC address ",
//					in->router->name, in->port);
//				print_mac(in->mac);
//				printf("\n");
				tricky.clear();
				for (ln3=newdata.routers.get_head(); ln3!=NULL; ln3=ln3->next)
				{
					r2 = (Router*) ln3->item;
					if (r2->type == 's')
					{
						d = get_device(r2);
						for (l=0; l<d->num_macs; l++)
						{
							if ((zero_mac(in->mac)==0) && (zero_mac(d->macs[l].mac)==0) &&
									(compare_mac(in->mac, d->macs[l].mac)==1))
							{
								// found a match so search for which interface on the switch
								// it was seen on
								for (ln4=r2->interfaces.get_head(); ln4!=NULL; ln4=ln4->next)
								{
									in4 = (Interface*) ln4->item;
									if (in4->index == d->macs[l].port)
									{
//										printf("  Switch %s / %s matches on bridging tables\n",
//											r2->name, in4->port);
										tricky.tag(in4);
									}
								}
							}
						}
					}
				}
				if (tricky.size()>0)
					build_tricky(in, &tricky);
//				printf("\n");
			}
		}
	}


// checking vlans

	for (ln=newdata.routers.get_head(); ln!=NULL; ln=ln->next)
	{
		r = (Router*) ln->item;
		for (ln2=r->interfaces.get_head(); ln2!=NULL; ln2=ln2->next)
		{
			in = (Interface*) ln2->item;
			if (in->vlan==1)
			{
//				printf("VLAN %s / %s\n", r->name, in->port);
			}else
			if ((strcmp(in->port, "sc0")==0) || (strcmp(in->port, "VL1")==0))
			{
//				printf("FAKE VLAN %s / %s\n", r->name, in->port);
// HACK
				in->vlan = 1;
			}
		}
	}

// checking levels

/*
	for (ln=newdata.routers.get_head(); ln!=NULL; ln=ln->next)
	{
		r = (Router*) ln->item;
		printf("LEVEL %d %s\n", r->selected, r->name);
	}
	for (ln=newdata.networks.get_head(); ln!=NULL; ln=ln->next)
	{
		n = (Network*) ln->item;
		if (n->interfaces.size()>1)
		{
			printf("LEVEL %d ", n->selected);
			for (ln2=n->interfaces.get_head(); ln2!=NULL; ln2=ln2->next)
			{
				in = (Interface*) ln2->item;
				printf("%s ", in->router->name);
			}
			printf("\n");
		}
	}
*/

}

void Topology::join_networks(Interface *in1, Interface *in2)
{
	Network *n1, *n2;
	Interface *in;
	Link *ln;

	n1 = in1->network;
	n2 = in2->network;
	for (ln=n1->interfaces.get_head(); ln!=NULL; ln=ln->next)
	{
		in = (Interface*) ln->item;
		n2->interfaces.tag(in);
		in->network = n2;
	}
	newdata.networks.del(n1);
	delete n1;
}

void Topology::build_tricky(Interface *in, List *s)
{
	Router *r, *r2;
	Device *d;
	int l, tag, count;
	Link *ln, *ln2, *ln3, *ln4;
	Interface *in2, *in3, *in4, *in5;
	List joins, swlst, otherends;
	Join *j;

	// if there is only one switch just attach it and return

	if (s->size()==1)
	{
		ln = s->get_head();
		join_networks(in, (Interface*)ln->item);
		return;
	}

	// start by building a list of links between switches

	joins.clear();

	for (ln=s->get_head(); ln!=NULL; ln=ln->next)
	{
		in4 = (Interface*) ln->item;
		r = in4->router;
		for (ln3=r->interfaces.get_head(); ln3!=NULL; ln3=ln3->next)
		{
			in2 = (Interface*) ln3->item;
			if (zero_mac(in2->mac)==0)
			{
				// we have a mac address to check for
				for (ln2=s->get_head(); ln2!=NULL; ln2=ln2->next)
				{
					in5 = (Interface*) ln2->item;
					r2 = in5->router;
					if (r!=r2)
					{
						d = get_device(r2);
						for (l=0; l<d->num_macs; l++)
						{
							if ((zero_mac(d->macs[l].mac)==0) &&
									(compare_mac(in2->mac, d->macs[l].mac)==1))
							{
								for (ln4=r2->interfaces.get_head(); ln4!=NULL; ln4=ln4->next)
								{
									in3 = (Interface*) ln4->item;
									if (in3->index == d->macs[l].port)
									{
										j = new Join();
										j->in1 = in2; j->in2 = in3;
										joins.tag(j);
//										printf("  Link %s / %s  ->  %s / %s\n",
//											r->name, in2->port, r2->name, in3->port);
									}
								}
							}
						}
					}
				}
			}
		}
	}

	// go through the joins table looking for switch interfaces that match
	// the switch list interfaces tagging those that match. Those that don't
	// match add to the router interface network

	swlst.clear();
	count = 0;
	for (ln=s->get_head(); ln!=NULL; ln=ln->next)
	{
		in2 = (Interface*) ln->item;
		tag = 0;
		for (ln2=joins.get_head(); ln2!=NULL; ln2=ln2->next)
		{
			j = (Join*) ln2->item;
			if ((in2 == j->in1) || (in2 == j->in2))
			{
				tag = 1;
			}
		}
		if (tag == 0)
		{
			join_networks(in, in2);
			swlst.tag(in2->router);
			in2->router->selected = 1;
		}else
			count++;
	}

	// if all switches match we don't have enough info to build properly

	if (count == s->size())
	{
		for (ln=s->get_head(); ln!=NULL; ln=ln->next)
		{
			in2 = (Interface*) ln->item;
// SERVER CRASH HERE
			join_networks(in, in2);
		}
	}

	// now add on all the leaf switches recursively from the trunk out

	for (ln=swlst.get_head(); ln!=NULL; ln=ln->next)
	{
		r = (Router*) ln->item;
//		printf("Processing swlst entry %s\n", r->name);
		otherends.clear();
		for (ln2=joins.get_head(); ln2!=NULL; ln2=ln2->next)
		{
			j = (Join*) ln2->item;
			if (j->in1->router == r)
			{
				// we have a match so check it is not already connected
				if (j->in1->network->interfaces.size()==1)	// not connected
					otherends.tag(j);
			}else
			if (j->in2->router == r)
			{
				if (j->in2->network->interfaces.size()==1)	// not connected
				{
					in2 = j->in1; j->in1 = j->in2; j->in2 = in2;
					otherends.tag(j);
				}
			}
		}
		// for each in otherends if matches switch list interface and
		// not already connected, connect to j->in1 and add switch
		// to end of swlst
		for (ln2=otherends.get_head(); ln2!=NULL; ln2=ln2->next)
		{
			j = (Join*) ln2->item;
			in2 = j->in2;
			in2->vlan = 1;
			count = 0;
			for (ln3=s->get_head(); ln3!=NULL; ln3=ln3->next)
			{
				in3 = (Interface*) ln3->item;
				if (in2==in3) { count++; in2->vlan = 0; }
			}
			if ((count>0) && (in2->network->interfaces.size()==1))
			{
				join_networks(j->in1, in2);
				in2->network->selected = r->selected+1;
				swlst.tag(in2->router);
				in2->router->selected = r->selected+1;
//				printf("    Adding %s to the swlst list\n", in2->router->name);
			}
			if (in2->network->interfaces.size()>1) in2->vlan = 0;
		}
	}

	// clean up the joins list

	for (ln=joins.get_head(); ln!=NULL; ln=ln->next)
	{
		j = (Join*) ln->item;
		delete j;
	}
	joins.clear();
}


/*****************************************************************************
 *
 *  This section adds and merges machines to the newdata structure
 *
 *****************************************************************************/

Machine *Topology::lookup_mac_machine(unsigned char *mac)
{
	Link *ln;
	Machine *m;

	for (ln=newdata.machines.get_head(); ln!=NULL; ln=ln->next)
	{
		m = (Machine*) ln->item;
		if (compare_mac(m->mac, mac)==1)
			return m;
	}
	return NULL;
}

Interface *Topology::lookup_mac_router(unsigned char *mac)
{
	Link *ln, *ln2;
	Router *r;
	Interface *in;

	for (ln=newdata.routers.get_head(); ln!=NULL; ln=ln->next)
	{
		r = (Router*) ln->item;
		for (ln2=r->interfaces.get_head(); ln2!=NULL; ln2=ln2->next)
		{
			in = (Interface*) ln2->item;
			if (compare_mac(in->mac, mac)==1)
				return in;
		}
	}
	return NULL;
}

// check for MAC in all interface address/bridging tables in network
int Topology::mac_match_all(Machine *m, Network *n)
{
	int j, tag, count;
	Link *ln;
	Router *r;
	Interface *in;
	Device *dv;

//	unsigned char tmpmac[6];
//	convert_mac_3("08002081b7c6", tmpmac);
//	if (compare_mac(tmpmac, m->mac)==1)
//		printf("Eller 08002081b7c6, mac_match\n");

	count = 0;
	for (ln=n->interfaces.get_head(); ln!=NULL; ln=ln->next)
	{
		in = (Interface*) ln->item;
		r = in->router;
		dv = get_device(r);
		tag=0;
		for (j=0; j<dv->num_addresses; j++)
		{
			if ((compare_mac(dv->addresses[j].mac, m->mac)) &&
					(dv->addresses[j].index == in->index))
				tag=1;
		}
		for (j=0; j<dv->num_macs; j++)
		{
			if ((compare_mac(dv->macs[j].mac, m->mac)) &&
					(dv->macs[j].port == in->index))
				tag=1;
		}
		if (tag==1) count++;
//		if (tag==1)
//		{
//			if (compare_mac(tmpmac, m->mac)==1)
//				printf("  Found in table on %s\n", r->name);
//		}
	}
	if (n->interfaces.size()==count) return 1;
	return 0;
}

// compare then move machine if need to
void Topology::move_machine(Machine *m1, Interface *in2)
{
	int s1, s2;
	Network *nw;

	// check this is not the same network
	if (m1->network == in2->network) return;

	// is it already in the most optimal place
	if (m1->place == 1) return;

	// is the new location the most optimal place
	if (mac_match_all(m1, in2->network)==1)
	{
		// move machine
		nw = (Network*) m1->network;
		nw->machines.del(m1);
		nw = (Network*) in2->network;
		nw->machines.tag(m1);
		m1->network = in2->network;
		m1->place = 1;
		return;
	}

	// work out how many routers and switches are
	// attached to the two networks
	s1 = m1->network->interfaces.size();
	s2 = in2->network->interfaces.size();
	if (s1<s2) return;

	// is the network further away from the router
	if ((s1==s2) && (m1->network->selected >= in2->network->selected)) return;


	// everything says the new location is more probable so swap them
	nw = (Network*) m1->network;
	nw->machines.del(m1);
	nw = (Network*) in2->network;
	nw->machines.tag(m1);
	m1->network = in2->network;
}

void Topology::add_machine(Interface *in3, unsigned char *mac, IPAddress ip)
{
	Machine *m;
	int l;

	m = new Machine();
	m->ip.set(ip);
	for (l=0; l<6; l++) m->mac[l] = mac[l];
	m->last_seen = in3->router->last_seen;
	m->network = in3->network;
	in3->network->machines.tag(m);
	newdata.machines.tag(m);

	if (mac_match_all(m, in3->network)==1) m->place = 1;

//	unsigned char tmpmac[6];
//	convert_mac_3("08002081b7c6", tmpmac);
//	if (compare_mac(tmpmac, mac)==1)
//		printf("Eller 08002081b7c6, creating new machine, place = %d\n", m->place);
}

void Topology::build_machines()
{
	Link *ln, *ln2;
	Machine *m;
	int j;
	Device *dv;
	Interface *in, *in2, *in3;
	IPAddress tmpip;
	Router *r;

	for (ln=newdata.routers.get_head(); ln!=NULL; ln=ln->next)
	{
		r = (Router*) ln->item;
		dv = get_device(r);
		for (j=0; j<dv->num_addresses; j++)
		{
			if (zero_mac(dv->addresses[j].mac)==0)
			{
				// get the interface the machine is connected to from the index
				in3 = NULL;
				for (ln2=r->interfaces.get_head(); ln2!=NULL; ln2=ln2->next)
				{
					in2 = (Interface*) ln2->item;
					if (dv->addresses[j].index == in2->index)
						in3=in2;
				}
				if ((in3!=NULL) && (in3->vlan==0))
				{
					// go through list of machines looking for a matching MAC
					in = lookup_mac_router(dv->addresses[j].mac);
					// if its a router then ignore it
					if (in==NULL)
					{
						m = lookup_mac_machine(dv->addresses[j].mac);
						if (m==NULL)	// no match so create a new machine
						{
							add_machine(in3, dv->addresses[j].mac, dv->addresses[j].ip);
						}else
						{
							move_machine(m, in3);
						}
					}
				}
			}
		}
		for (j=0; j<dv->num_macs; j++)
		{
			if (zero_mac(dv->macs[j].mac)==0)
			{
				in3 = NULL;
				for (ln2=r->interfaces.get_head(); ln2!=NULL; ln2=ln2->next)
				{
					in2 = (Interface*) ln2->item;
					if (dv->macs[j].port == in2->index)
						in3=in2;
				}
				if ((in3!=NULL) && (in3->vlan==0))
				{
					in = lookup_mac_router(dv->macs[j].mac);
					// if MAC of a device ignore it
					if (in==NULL)
					{
						m = lookup_mac_machine(dv->macs[j].mac);
						if (m==NULL)
						{
							tmpip.clear();
							add_machine(in3, dv->macs[j].mac, tmpip);
						}else
						{
							move_machine(m, in3);
						}
					}
				}
			}
		}
	}
}

/*****************************************************************************
 *
 *  This section merges the two data structures into a single topology diagram
 *
 *****************************************************************************/

Subnet *Topology::find_subnet(IPAddress ip)
{
	Link *ln;
	Subnet *sn;

	for (ln=newdata.subnets.get_head(); ln!=NULL; ln=ln->next)
	{
		sn = (Subnet*) ln->item;
		if (in_subnet(ip, sn->ip, sn->mask))
			return sn;
	}
	return NULL;
}

void Topology::merge_networks(Interface *in1, Interface *in2)
{
	Link *ln, *ln2, *ln3;
	Interface *in, *in3;
	Router *r;

	// in1 is the interface on the new device
	// in2 is the equivalent in the old setup which is connected

	// find an interface in old to the network that also exists in new that is
	// not the new network network, the network,

	for (ln=in2->network->interfaces.get_head(); ln!=NULL; ln=ln->next)
	{
		in = (Interface*) ln->item;
		if (in!=in2)
		{
			// in is an interface in old, so find the equivalent in new
			for (ln2=newdata.routers.get_head(); ln2!=NULL; ln2=ln2->next)
			{
				r = (Router*) ln2->item;
				for (ln3=r->interfaces.get_head(); ln3!=NULL; ln3=ln3->next)
				{
					in3 = (Interface*) ln3->item;
					if ((in3->index == in->index) &&
							(strcmp(in->router->name, r->name)==0))
					{
						// merge in1->network and in3->network (throw away in1->network)
						if (in1->network != in3->network)
						{
							newdata.networks.del(in1->network);
							delete (in1->network);
							in1->network = in3->network;
							in3->network->interfaces.tag(in1);
						}
					}
				}
			}
		}
	}
}

Network *Topology::lookup_machine(Machine *m)
{
	Link *ln, *ln2, *ln3;
	Interface *in, *in3;
	Router *r;

	ln = m->network->interfaces.get_head();
	in = (Interface*) ln->item;

	for (ln2=newdata.routers.get_head(); ln2!=NULL; ln2=ln2->next)
	{
		r = (Router*) ln2->item;
		if (strcmp(in->router->name, r->name)==0)
		{
			for (ln3=r->interfaces.get_head(); ln3!=NULL; ln3=ln3->next)
			{
				in3 = (Interface*) ln3->item;
				if (in3->index == in->index)
				{
					return in3->network;
				}
			}
		}
	}
	return NULL;
}

void Topology::merge()
{
	char buf[256];
	Link *ln, *ln2, *ln3, *ln4;
	IPAddress *tip, *tip2;
	Interface *in, *in2;
	Machine *m, *m2, *m3;
	Network *n;
//	FILE *f;
	int l;
	Router *r, *r2;

	/*************************************************************************
		We have a newdata structure built, so load olddata structure. All
		we are going to do is make changes to the newdata structure.
	*************************************************************************/

	sprintf(buf, "%s/network", mapdir);
/*
	f = fopen(buf, "r");
	if (!f) return;
	olddata.load(f);
	fclose(f);
*/
	olddata.loadxml(buf);

	/************************************************************************
		If any routers have no interfaces, i.e. a router or switch that we
		could not contact with snmp, then copy the router details and
		interfaces, then create new networks.
	************************************************************************/

// a router or switch will only exist if it is in the devices file, if the
// device is removed from service it will not appear in the router list

	for (ln=newdata.routers.get_head(); ln!=NULL; ln=ln->next)
	{
		r = (Router*) ln->item;
		// if a machine fails to be loaded it is tagged unknown type
		if ((r->type == 'u') && (r->interfaces.size() == 0))
		{
			for (ln2=olddata.routers.get_head(); ln2!=NULL; ln2=ln2->next)
			{
				r2 = (Router*) ln2->item;
				if ((strcmp(r->name, r2->name)==0) && (r2->interfaces.size()>0))
				{
					// copy details from r2 to r
					// name
					strcpy(r->community, r2->community);
					r->pingname.set(r2->pingname);
					// lastseen
					r->type = r2->type;
					for (ln3=r2->interfaces.get_head(); ln3!=NULL; ln3=ln3->next)
					{
						in2 = (Interface*) ln3->item;
						in = new Interface();
						in->index = in2->index;
						for (l=0; l<6; l++) in->mac[l] = in2->mac[l];
						in->speed = in2->speed;
						in->type = in2->type;
						in->status = in2->status;
						strcpy(in->descr, in2->descr);
						strcpy(in->port, in2->port);
						for (ln4=in2->ip_addresses.get_head(); ln4!=NULL; ln4=ln4->next)
						{
							tip = (IPAddress*) ln4->item;
							tip2 = new IPAddress();
							tip2->set(*tip);
							in->ip_addresses.tag(tip2);
						}
						in->router = r;
						r->interfaces.tag(in);

						n = new Network();
						n->interfaces.tag(in);
						in->network = n;
						newdata.networks.tag(n);

						// router r now has all it's bits now we need to attach it
						// we don't need to know all devices have been created as they
						// will be attached in the same way when they are created
						if (in2->network!=NULL)
							if (in2->network->interfaces.size()>1)
								merge_networks(in, in2);
					}
				}
			}
		}
	}

	/************************************************************************
		Now we have all routers and switches attached as before, scan the list
		of machines and lookup in the new structure a match, if it is not there
		then make a new machine at the equivalent location with all the old details
	************************************************************************/

	for (ln=olddata.machines.get_head(); ln!=NULL; ln=ln->next)
	{
		m = (Machine*) ln->item;
		// lookup equivalent location for m in new
		n = lookup_machine(m);
		m3 = NULL;
		for (ln2=newdata.machines.get_head(); ln2!=NULL; ln2=ln2->next)
		{
			m2 = (Machine*) ln2->item;
			if (compare_mac(m->mac, m2->mac)==1)
				m3 = m2;
		}
		if (m3!=NULL)
		{
			// found the same machine, so work out which is the best location
			// m is the old machine, m3 is the new machine
			if ((m3->place==0) && (m->place==1))
			{
				// n is the network to attach to in new
				if (n!=NULL)
				{
					m3->network->machines.del(m3);
					m3->network = n;
					m3->network->machines.tag(m3);
					m3->place = 1;
				}
			}
		}else
		{
			// failed to find a match so copy over old machine
			// m is the old machine
			if (n!=NULL)
			{
				m3 = new Machine();
				strcpy(m3->name, m->name);
				m3->ip.set(m->ip);
				m3->last_seen = m->last_seen;
				for (l=0; l<6; l++) m3->mac[l] = m->mac[l];
				m3->place = m->place;
				m3->network = n;
				m3->network->machines.tag(m3);
				newdata.machines.tag(m3);
			}
		}
	}

	/************************************************************************
		Check the timestamps, if a machine has not been seen in x days then
		remove it.
	************************************************************************/

/*
	for (ln=newdata.machines.get_head(); ln!=NULL; ln=ln->next)
	{
		m = (Machine*) ln->item;
//	if (m->last_seen older than 3 months)
		{
			// delete machine
		}
	}
*/
}

void Topology::merge_machines_file()
{
	FILE *f;
	char buf[256];
	char mname[256];
	char mmac[256];
	char mip[256];
	char mcontact[256];
	char mlocation[256];
	char mdescr[256];
	Machine *m;
	Router *r;
	Link *ln;

	unsigned char tmpmac[6];
	IPAddress tmpip;

	sprintf(buf, "%s/machines", mapdir);
	f = fopen(buf, "r");
	if (f)
	{
		while(fscanf(f, "%[^,],%[^,],%[^,],%[^,],%[^,],%[^\n]\n",
			mname, mmac, mip, mcontact, mlocation, mdescr)==6)
		{
			convert_mac_3(mmac, tmpmac);
			tmpip.set(mip);
			for (ln=newdata.machines.get_head(); ln!=NULL; ln=ln->next)
			{
				m = (Machine*) ln->item;
				if (compare_mac(m->mac, tmpmac)==1)
				{
					m->registered = 1;
					strcpy(m->name, mname);
					strcpy(m->location, mlocation);
					strcpy(m->contact, mcontact);
					strcpy(m->descr, mdescr);
					if (m->ip.zero())
					{
						m->ip.set(tmpip);
					}
				}else
				if ((zero_mac(tmpmac)==1) && (!tmpip.zero()) && (!m->ip.zero()))
				{
					if (tmpip.compare(m->ip)==1)
					{
						m->registered = 1;
						strcpy(m->name, mname);
						strcpy(m->location, mlocation);
						strcpy(m->contact, mcontact);
						strcpy(m->descr, mdescr);
					}
				}
			}

			for (ln=newdata.routers.get_head(); ln!=NULL; ln=ln->next)
			{
				r = (Router*) ln->item;
				if (strcmp(r->name, mname)==0)
				{
					r->registered = 1;
					strcpy(r->location, mlocation);
					strcpy(r->contact, mcontact);
					strcpy(r->descr, mdescr);
				}
			}
		}
		fclose(f);
	}
}

void Topology::merge_subnets_file()
{
	char buf[256], bufip[256], bufmask[256], bufname[256];
	IPAddress tmpip;
	Subnet *sn;
	FILE *f;
	int msk;
	Link *ln;

	sprintf(buf, "%s/subnets", mapdir);
	f = fopen(buf, "r");
	if (f)
	{
		while(fscanf(f, "%[^/]/%[^,],%[^\n]\n", bufip, bufmask, bufname)==3)
		{
			tmpip.set(bufip);
			sscanf(bufmask, "%d", &msk);
			for (ln=newdata.subnets.get_head(); ln!=NULL; ln=ln->next)
			{
				sn = (Subnet*) ln->item;
				if ((sn->mask == msk) && (tmpip.compare(sn->ip)))
					strcpy(sn->name, bufname);
			}
		}
		fclose(f);
	}
}

void Topology::save()
{
	Link *ln, *ln2, *ln3, *ln4;
	Machine *m;
	Network *n;
	Interface *in;
	Subnet *sn;
	int flag;
//	FILE *f;
	char buf[256];
	IPAddress tmpip, *tip;

	// add inventory stuff for machine names and subnets
	merge_machines_file();
	merge_subnets_file();

	/***************************************************************************
	Create subnet entries for each network object
	***************************************************************************/

	for (ln=newdata.networks.get_head(); ln!=NULL; ln=ln->next)
	{
		n = (Network*) ln->item;
		for (ln2=n->interfaces.get_head(); ln2!=NULL; ln2=ln2->next)
		{
			in = (Interface*) ln2->item;
			for (ln4=in->ip_addresses.get_head(); ln4!=NULL; ln4=ln4->next)
			{
				tip = (IPAddress*) ln4->item;
				flag = 0;
				for (ln3=n->subnets.get_head(); ln3!=NULL; ln3=ln3->next)
				{
					sn = (Subnet*) ln3->item;
					if (in_subnet(*tip, sn->ip, sn->mask)==1) flag = 1;
				}
				if (flag == 0)
				{
					sn = find_subnet(*tip);
					if (sn) n->subnets.tag(sn);
				}
			}
		}
		for (ln2=n->machines.get_head(); ln2!=NULL; ln2=ln2->next)
		{
			m = (Machine*) ln2->item;
			flag = 0;
			for (ln3=n->subnets.get_head(); ln3!=NULL; ln3=ln3->next)
			{
				sn = (Subnet*) ln3->item;
				if (in_subnet(m->ip, sn->ip, sn->mask)==1) flag = 1;
			}
			if (flag == 0)
			{
				sn = find_subnet(m->ip);
				if (sn) n->subnets.tag(sn);
			}
		}
	}

	/******************************************
	save newdata
	******************************************/

	sprintf(buf, "%s/network.new", mapdir);
	newdata.savexml(buf);
	sprintf(buf, "mv %s/network.new %s/network", mapdir, mapdir);
	system(buf);

/*
	f = fopen(buf, "w");
	if (f)
	{
		newdata.save(f);
		fclose(f);
		sprintf(buf, "mv %s/network.new %s/network", mapdir, mapdir);
		system(buf);
	}
*/
}

